/*
 * Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
 *
 * This file is part of Qt Web Runtime.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifdef Q_OS_MAEMO5
#include <QMaemo5InformationBox>
#endif

#include <QCoreApplication>

#include "widgetinstallerview.h"
#include "WgtWidget.h"
#include "WidgetManager.h"


WidgetInstallerView::WidgetInstallerView(WidgetManager *widgetMgr, QObject *parent)
    : QObject(parent), m_widgetMgr(widgetMgr), m_result(0), m_silentMode(false)
{
    // do nothing
}

WidgetInstallerView::~WidgetInstallerView()
{
    // do nothing
}

void WidgetInstallerView::slotInstallationState(WidgetInstallError error)
{
    m_result = error;

    QString message;
    switch (error) {
    case WidgetCorrupted:
    case WidgetUnZipBundleFailed:
        message = qtTrId("Installation of web application failed - invalid package.");
        break;

    case WidgetTypeValidationFailed:
        message = qtTrId("Installation of web application failed - invalid package.");
        break;

    case WidgetParseManifestFailed:
        message = qtTrId("Installation of web application failed - invalid package.");
        break;

    case WidgetInsufficientDiskSpace:
        message = qtTrId("Installation of web application failed - out of space.");
        break;

    case WidgetStartFileNotFound:
        message = qtTrId("Installation of web application failed - no start file.");
        break;

    case WidgetPlatformSpecificInstallFailed:
        message = qtTrId("Installation of web application failed - can't generate the Debian package.");
        break;

    case WidgetCapabilityNotAllowed:
         message = qtTrId("Installation of web application failed - capability not allowed.");
         break;

    case WidgetRmDirFailed:
    case WidgetReplaceFailed:
    case WidgetRegistrationFailed:
        message = qtTrId("Installation of web application failed - registration failed.");
        break;

    case WidgetFindSignatureFailed:
    case WidgetInstallFailed:
    case WidgetSharedLibraryNotSigned:
        message = qtTrId("Installation of web application failed - shared libs must be signed properly.");
        break;

    case WidgetIdInvalid:
        message = qtTrId("Installation of web application failed - ID is invalid.");
        break;

    default:
        qCritical() << "Unknown widget installation error code:" << error;
        break;
    }

    qWarning() <<"Widget installation error:"  << message;

    if (!m_silentMode) {
#ifdef Q_OS_MAEMO5
        QMaemo5InformationBox::information(0, message);
#endif
    }
}

void WidgetInstallerView::slotInstallationSucceed()
{
    m_result = WidgetInstallSuccess;
    exitApplication();
}


void WidgetInstallerView::exitApplication()
{
    if(m_result>=0)
        QCoreApplication::exit(m_result);
}

void WidgetInstallerView::slotCancelInstallation()
{
    emit installationCancelled();
}

void WidgetInstallerView::slotAboutToReplaceExistingWidget()
{
    m_widgetMgr->tryContinueInstallation();
}

void WidgetInstallerView::slotAboutToInstallWidgetWithFeatures(QList<QString> capList)
{
    //TODO: if we need to show some UI control with list of granted features, this is the place
    Q_UNUSED(capList);

    m_widgetMgr->tryContinueInstallation();
}

void WidgetInstallerView::slotAboutToInstallUntrustedWidget(WidgetInstallError error)
{
    Q_UNUSED(error);
    
    m_widgetMgr->tryContinueInstallation();
}

int WidgetInstallerView::result() const
{
    return m_result;
}

void WidgetInstallerView::setSilentMode(bool mode)
{
    m_silentMode = mode;
}
