/*
 * Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
 *
 * This file is part of Qt Web Runtime.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#include "zzip.h"
#include "unzip.h"
#include <QDebug>
#include <QDir>
#include <QFileInfo>
#include <QFile>

int Zzip::unzip(const QString &path, const QString &unZipPath, unsigned long &size)
{
    unz_global_info globalInfo;
    unzFile ufile = unzOpen(path.toLocal8Bit().data());
    if (unzGetGlobalInfo(ufile, &globalInfo)) {
        qCritical() << "Failed to extract the widget: unable to get information of " << path;
        unzClose(ufile);
        return -1;
    }

    size = 0;
    QDir destDir(unZipPath);
    char fName[256];
    unz_file_info finfo;
    QString fileName, dirName;
    for (unsigned int i = 0; i < globalInfo.number_entry; ) {
        if (UNZ_OK != unzGetCurrentFileInfo(ufile, &finfo, fName, 256, NULL, 0, NULL, 0)) {
            qCritical() << "Failed to extract files from the widget: unable to get the information of the file for unzip";
            unzClose(ufile);
            return -1;
        }

        fileName = destDir.absolutePath() + "/" + QString(fName);
        dirName = fileName.left(fileName.lastIndexOf('/'));
        if (!dirName.isEmpty() && !QFileInfo(dirName).exists()) {
            if (!destDir.mkpath(dirName)) {
                qCritical() << "Failed to extract files from the widget: unable to create directory " << dirName;
                unzClose(ufile);
                return -1;
            }
        }

        if (QFileInfo(fileName).isDir()) {
            int result = unzGoToNextFile(ufile);

            if (UNZ_END_OF_LIST_OF_FILE == result) {
                unzClose(ufile);
                return 0;
            }

            if (UNZ_OK != result) {
                qCritical() << "Failed to extract files from the widget: unable to get the information for the next file";
                unzClose(ufile);
                return -1;
            }
            continue;
        }

        size += finfo.uncompressed_size;
        QByteArray content(finfo.uncompressed_size, 0);
        if (UNZ_OK != unzOpenCurrentFile(ufile)
            || 0 > unzReadCurrentFile(ufile, content.data(), content.size())
            || UNZ_OK != unzCloseCurrentFile(ufile)) {
            qCritical() << "Failed to extract files from the widget: unable to extract file from the widget";
            unzClose(ufile);
            return -1;
        }

        QFile newFile(fileName);
        if (!newFile.open(QIODevice::WriteOnly)
            || -1 == newFile.write(content)) {
            qCritical() << "Failed to extract files from the widget: unable to write the extracted file to " << fileName;
            unzClose(ufile);
            return -1;
        }
        newFile.close();

        if (++i < globalInfo.number_entry) {
            int result = unzGoToNextFile(ufile);

            if (UNZ_END_OF_LIST_OF_FILE == result) {
                unzClose(ufile);
                return 0;
            }

            if (UNZ_OK != result) {
                qCritical() << "Failed to extract files from the widget: unable to switch to the next file";
                unzClose(ufile);
                return -1;
            }
        }
    }

    unzClose(ufile);

    return 0;
}


unsigned long Zzip::uncompressedSize(const QString &path)
{
    unsigned long size = 0;

    unz_global_info globalInfo;
    unzFile ufile = unzOpen(path.toLocal8Bit().data());
    if (unzGetGlobalInfo(ufile, &globalInfo)) {
        qWarning() << "Zzip::uncompressedSize(): unzGetGlobalInfo() failed";
        unzClose(ufile);
        return -1;
    }

    char fName[256] = {0};
    unz_file_info finfo;
    QString fileName;

    for (unsigned int i = 0; i < globalInfo.number_entry; ) {
        unzGetCurrentFileInfo(ufile, &finfo, fName, 256, NULL, 0, NULL, 0);

        if (QFileInfo(fileName).isDir()) {
            unzGoToNextFile(ufile);
            continue;
        }

        size += finfo.uncompressed_size;

        if (++i < globalInfo.number_entry)
            unzGoToNextFile(ufile);
    }

    unzClose(ufile);

    return size;
}
