# Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
#
# This file is part of Qt Web Runtime.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# version 2.1 as published by the Free Software Foundation.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
#
#

####################################################################################
#
# Common Web Run Time - common settings - cwrt.pri
#
# This project include by any cWRT component that utilizes common settings.
# It should not be included directly in case the component has dependencies on
# webkit - cwrt-webkit.pri must be included instead.
#
####################################################################################

isEmpty(_CWRT_PRI_INCLUDED_): {
    _CWRT_PRI_INCLUDED_=1

    #####################################################
    # Initialize the feature and build flags for platforms
    #####################################################
    unix:!symbian {
        DEFINES += "QTWRT_ENABLE_UNIX_CERTIFICATE_MANAGER=1"
    }

    ###################################################
    # Use the "what" to enable features and build flags
    ###################################################
    contains( what, contextmenu ) {
        DEFINES *= QTWRT_ENABLE_CONTEXT_MENU
    }

    contains( what, pauseresumenavigation ) {
        DEFINES *= QTWRT_PAUSE_DOM_OBJECTS
    }

    contains( what, email ) {
        DEFINES *= QTWRT_ENABLE_EMAIL
    }

    contains( what, graphicsview ) {
        DEFINES *= QTWRT_ENABLE_GRAPHICSWEBVIEW
    }

    contains( what, orbit ) {
        CONFIG *= Hb
        DEFINES *= QTWRT_USE_ORBIT
    }

    contains( what, embeddedinstaller ) {
        DEFINES *= QTWRT_USE_EMBEDDEDINSTALLER
    }

    contains( what, s60widgetregistry ) {
        DEFINES *= QTWRT_USE_QT_PAUSE_DOM_OBJECTS
    }

    contains( what, usif ) {
        DEFINES *= QTWRT_USE_USIF
    }

    contains( what, fmfilemanager ) {
        DEFINES *= QTWRT_USE_FMFILEMANAGER
    }

    contains ( what, uselocalsha256 ) {
        DEFINES *= QTWRT_USE_LOCAL_SHA256
    }

    contains ( what, enableunsignedwidgets ) {
        DEFINES *= QTWRT_ENABLE_UNSIGNED_WIDGETS
    }

    contains( what, enablefastallocator ) {
        CONFIG += QTWRT_ENABLE_CUSTOM_MEMORY_ALLOCATOR
    }

    contains ( what, unreleasedqtmobility ) {
        DEFINES *= QTWRT_USE_UNRELEASEDQTMOBILITY
    }

    contains ( what, desktopfilewriter ) {
        DEFINES *= QTWRT_USE_DESKTOPFILEWRITER
    }

    CONFIG(debug, debug|release) {
        SUBDIRPART = Debug
    } else {
        SUBDIRPART = Release
    }

    # wrt_tracing flag may be enabled before final production runs
    contains( wrt_tracing, on ) {
        DEFINES += ENABLE_TRACE_SIGNAL
    }

    CWRT_HOME=$$PWD

    # Figure out the root of where stuff should go (this could be done via configure)
    OUTPUT_DIR = $$PWD/..

    OBJECTS_DIR = $$OUTPUT_DIR/WrtBuild/$$SUBDIRPART/$$TARGET/tmp
    MOC_DIR = $$OUTPUT_DIR/WrtBuild/$$SUBDIRPART/$$TARGET/tmp
    RCC_DIR = $$OUTPUT_DIR/WrtBuild/$$SUBDIRPART/$$TARGET/tmp
    UI_DIR = $$OUTPUT_DIR/WrtBuild/$$SUBDIRPART/$$TARGET/tmp
    TEMPDIR= $$OUTPUT_DIR/WrtBuild/$$SUBDIRPART/$$TARGET/build
    DESTDIR = $$OUTPUT_DIR/WrtBuild/$$SUBDIRPART/bin
    QMAKE_RPATHDIR = $$DESTDIR $$QMAKE_RPATHDIR
    QMAKE_LIBDIR = $$DESTDIR $$QMAKE_LIBDIR

    INCLUDEPATH+=$$CWRT_HOME

    # Remove guard as it should not be needed
    !symbian {
        include(../shared.pri)
    }

    # Backwards compatibility in current build system
    maemo {
         maemo5 {
            CONFIG-=maemo6
        } else {
            CONFIG+=maemo6
        }
        include(cwrt-maemo.pri)
    }

    # Add the output dirs to the link path too
    LIBS += -L$$DESTDIR

    DEPENDPATH += .
    INCLUDEPATH += .

    maemo {
        CWRT_INCLUDE = $$PWD/../src/common/ipc/serviceipc $$PWD/../src/common/ipc/serviceipcserver $$PWD/../src/common/security $$PWD/serviceframework/api $$PWD/../src/common/securestorage
    }

    # Currently only used for Linux and Maemo platforms. Please if-else if this needed for Symbian.
    # When using this component should concatenate component specific header directory.
    # This is commented out as we should use CWRT_INSTALL_INC_DIR defined in ths shared.pri file
    # CWRT_INSTALL_PUBLIC_HEADERS_DIR = /usr/include/cwrt

    !win32-msvc*:QMAKE_CXXFLAGS_DEBUG += -g

    CONFIG(gcov)   {
       LIBS           += -lgcov
       QMAKE_CXXFLAGS += -fprofile-arcs -ftest-coverage
    }
}
