/*
 * Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
 *
 * This file is part of Qt Web Runtime.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

/**
 * @dataschema
 * @class
 * @description The InterfaceInfo object contains summary information about a {@link wrt.device.ServiceInterface}. Each 
 * service interface is uniquely identified by its name and version number.
 */
wrt.device.InterfaceInfo = function()   {
    /**
    * @description Specifies the fully qualified name of the service interface. The name can be, for example, 
    * "calendar", "contacts", or "media". This is the name of the service object that provides access to the 
    * corresponding platform service.
    *
    * @type String
    */
    this.name    = null;
    /**
    * @description Specifies the version of the service interface. The version can be, for example, 2.0, 2.1, or 3.0.
    * Note: To ensure that a device supports all methods of an API version, see Using a Platform Services 2.0 API in a widget.
    *
    * @type Number
    */
    this.version = null;
}

/**
 * @dataschema
 * @class
 * @description The ServiceInterface object represents an instance of a service interface: a service object. 
 * This service object is only used in Platform Services 2.0. Each service object contains a set of methods 
 * and properties that provide access to a platform service. The exact set of methods and properties depends 
 * on the service and the service object. For more information about the service objects, see the Platform 
 * Services 2.0 API reference. </br>
 * Service objects are not constructed directly. Instead, to create a service object, use the {@link wrt.device.load} method.
 *
 */
wrt.device.ServiceInterface = function()   {
    /**
    * @description Specifies the fully qualified name of the service interface. The name can be, for example, "calendar", 
    * "contacts", or "media". This is the name of the service object that provides access to the corresponding platform service.
    *
    * @type String
    */
    this.interfaceName = null;
    /**
    * @description Specifies the version of the service interface. The version can be, for example, 2.0, 2.1, 2.2 and 3.0.
    * @type Number
    */
    this.version = null;
    /**
    * @description method defined in an service interface object. eg. contacts.getList()
    */
    this.functionXYZ = function(){
    }
}

/**
 * @dataschema
 * @class
 * @description The SystemProperties object contains information about system-wide properties whose allowed values 
 * can vary across device platforms. When you retrieve this object, the information it contains describes the value 
 * limitations on the current platform
 */
wrt.device.SystemProperties = function()   {
    /**
    * @description Specifies the minimum and maximum lengths for any URI string. The lengths are indicated in number of characters.
    * @type wrt.device.MinMax
    */
    this.uriPath = null;
}

/**
 * @dataschema
 * @class
 * @description The SystemProperties object contains information about system-wide properties whose allowed values 
 * can vary across device platforms. When you retrieve this object, the information it contains describes the value 
 * limitations on the current platform
 */
wrt.device.MinMax = function()   {
    /**
    * @description Specifies the minimum and maximum lengths for any URI string. The lengths are indicated in number of characters.
    * @type Number
    */
    this.min = null;
    /**
    * @description Specifies the minimum length for a URI string.
    * @type Number
    */
    this.max = null;
}
