#!/usr/bin/perl
use Cwd;
use File::Path;
use File::Copy;

#constants
use constant CONFIG_FILE_PATH   => '/config/config.conf';
use constant PROVIDER           => 'PROVIDER';
use constant WGZ_PATH           => 'WGZ_PATH';
use constant WGT_PATH           => 'WGT_PATH';
use constant TMP_PATH           => 'TMP_PATH';
use constant REL_PATH           => 'REL_PATH';

use constant FRAMEWORK_JS       => 'FRAMEWORK_JS';
use constant COMBINED_JS        => 'COMBINED_JS';
use constant TESTTREE_JS        => 'TESTTREE_JS';
use constant DEVICE_JS          => 'DEVICE_JS';
use constant EXPORT_PATH        				 => '/epoc32/winscw/c/private/200267C0/webapps/lib/wrt/device/';
use constant EXPORT_PATH_3RDPARTY        => '/epoc32/winscw/c/private/200267D6/webapps/lib/wrt/device/';
use constant OLD_EXPORT_PATH    				 => '/epoc32/winscw/c/private/200267C0/webapps/lib/nokia/device/';
use constant OLD_EXPORT_PATH_3RDPARTY    => '/epoc32/winscw/c/private/200267D6/webapps/lib/nokia/device/';


use constant TRUE               => 1;
use constant FALSE              => 0;

%options = (
    'cleanup'   => \&cmdCleanup,
    'widgets'   => \&cmdWidgets,
    'merge'     => \&cmdMerge,
    'maemo'     => \&cmdMaemo,
    'maemo-new' => \&cmdMaemoNew,
    'linux'     => \&cmdLinux,
    's60-new'   => \&cmdS60New,
    'symbian'   => \&cmdSymbian 
);

#variables
@providers = ();
$ziptool = "7z a -tzip ";
$compress = TRUE;
$emptypsfile = TRUE;
$doexport   = FALSE;

readConfig();

$option = shift;
if( length($option) > 0 )
{
    while( length($option) > 0 ) {
        if( defined $options{$option} ) {
            $options{$option}->();
        }
        
        $option = shift;
    }
}
else { 
    cleanup();
    merge();
    widgets();
}

#parse the input options
$option = shift;
while( length($option) > 0 )
{
    if( defined $options{$option} ) {
        $options{$option}->();
    }
    
    $option = shift;
}

sub cmdMaemo()
{
		cmdMaemoNew();
}

sub cmdMaemoNew()
{
    $ziptool = "zip ";
    $compress = FALSE;
    $doexport   = FALSE;
    cleanup();
    mergeForMaemo();
    widgets();
}

sub cmdLinux()
{
    $ziptool = "zip ";
    $compress = FALSE;
    $doexport = FALSE;
    cleanup();
    merge();
}

sub cmdS60New()
{
    $ziptool = "7z a -tzip ";
    $compress = TRUE;
    $doexport   = TRUE;
    cleanup();
    merge();
    widgets();
}

sub cmdSymbian()
{
    $compress = TRUE;
    $doexport   = TRUE;
    cleanup();
    merge();
}

sub readConfig()
{
    # This piece of code will get the physical path of this script
    # irrespective of from where this script is executed.
    use File::Spec::Functions qw(rel2abs);
    use File::Basename;
    
    $path = dirname(rel2abs($0));
    
    my $configfile = $path . CONFIG_FILE_PATH;
    
    my %config;
    open my $config, '<', $configfile or die $!;
    
    while (<$config>) {
        chomp;                  # no newline
        s/#.*//;                # no comments
        s/^\s+//;               # no leading white
        s/\s+$//;               # no trailing white
        next unless length;     # anything left?
        my ($var, $value) = split(/\s*=\s*/, $_, 2);
        if( exists $User_Preferences{$var} )
        {
            $User_Preferences{$var} = $User_Preferences{$var} . ' ' . $value;
        }
        else
        {
            $User_Preferences{$var} = $value;
        }
    }
    
    if( exists $User_Preferences{PROVIDER})
    {
        @providers = split(' ', $User_Preferences{PROVIDER});
    }
}

sub cmdCleanup()
{
    cleanup();
}

sub cleanup()
{
    rmtree( $path . $User_Preferences{WGT_PATH} );
    rmtree( $path . $User_Preferences{WGZ_PATH} );
    rmtree( $path . $User_Preferences{TMP_PATH} );

    rmtree( $path . $User_Preferences{REL_PATH} );
}
   
sub cmdMerge()
{
    cleanup();
    merge();
}

sub mergeForMaemo()
{
	  #create release directory
    mkdir($path . $User_Preferences{REL_PATH});
    #create directory for uncompressed files.
    mkdir($path . $User_Preferences{UNC_PATH});

		#Create empty platformservices.js for CP binary compatability.
    unlink( $path . $User_Preferences{REL_PATH} . "/" . $User_Preferences{COMBINED_JS} );
    open EMPTYFILE, ">", $path . $User_Preferences{UNC_PATH} . "/" . $User_Preferences{COMBINED_JS};
    close EMPTYFILE;


    # Merge all the individual modules into single module js file
    # place them in UNC_PATH directory
    foreach $provider(@providers) {
        #get module directory & open
       
        $providerjspath = "";
        if( $provider eq "sysinfo" )
        {
          $providerjspath = $path . "/" . $provider . "/platform/maemo";
        }
        else
        {
          $providerjspath = $path . "/" . $provider;
        } 
        opendir(DIR, $providerjspath );
        #create result combined file for this module. 
        my $tmpfile =  ">" . $path . $User_Preferences{UNC_PATH} . "/" . $provider . ".js";
        open PROVIDER_JS_FILES, $tmpfile or die $!; 
        while( $file = readdir(DIR) )
        {
            #process only .js files.
            if( rindex($file, ".js") == -1)
            {
                next;
            }
            #copy _data.js files to uncompressed dir.
            if( !(rindex($file, "_data") == -1) )
            {
                copy( $providerjspath . "/" . $file, $path . $User_Preferences{UNC_PATH} . "/" . $file);
                next;
            }
            open (PROVIDERFILE, $providerjspath . "/" . $file);
            while (<PROVIDERFILE>) {
                chomp;
                print PROVIDER_JS_FILES "$_\n";
           }
           close (PROVIDERFILE);
        }
        closedir(DIR);
        close PROVIDER_JS_FILES;
    }

    $jspath = $path . $User_Preferences{UNC_PATH};
    @jsfiles = ();
    opendir(DIR, $jspath) or die "Can't Open Directory $dirname: $!";

    if($doexport == TRUE){
        mkpath(EXPORT_PATH);
        mkpath(EXPORT_PATH_3RDPARTY);
        $EXPORT_PATH_DATA = EXPORT_PATH;
        $EXPORT_PATH_DATA_3RDPARTY = EXPORT_PATH_3RDPARTY;
        print("Exporting compressed javascript to wrt shared library...\n");
        print("------------------------------------------------------------\n");
    }

    while( $file = readdir(DIR)  ) {
        #Allow only .js files and skip _data.js files.
        if( rindex($file, ".js") == -1 || !(rindex($file, "_data") == -1) ) { next; }
        if( $compress == TRUE )    {
            system("java -jar $path/../../../tools/packaging/yuicompressor-2.4.2.jar $path$User_Preferences{UNC_PATH}/$file -o $path$User_Preferences{REL_PATH}/$file");
        }
        else{
            copy( $path . $User_Preferences{UNC_PATH} . "/" . $file, $path . $User_Preferences{REL_PATH} . "/" . $file );
        }
        if($doexport == TRUE){
            # EXPORT COMPRESSED JAVASCRIPT.
            copy( $path . $User_Preferences{REL_PATH} . "/" . $file, EXPORT_PATH . $file);
            copy( $path . $User_Preferences{REL_PATH} . "/" . $file, EXPORT_PATH_3RDPARTY . $file);
            print("copied: $EXPORT_PATH_DATA$file\n");
            print("copied: $EXPORT_PATH_DATA_3RDPARTY$file\n");
        }
    }
    closedir(DIR);
    
    if($doexport == TRUE){
        print("\nExporting compressed javascript to nokia shared library...\n");
        print("------------------------------------------------------------\n");
        mkpath(OLD_EXPORT_PATH);
        mkpath(OLD_EXPORT_PATH_3RDPARTY);
        $EXPORT_PATH_DATA = OLD_EXPORT_PATH;
        $EXPORT_PATH_DATA_3RDPARTY = OLD_EXPORT_PATH_3RDPARTY;
        copy( $path . $User_Preferences{REL_PATH} . "/device.js", OLD_EXPORT_PATH . "device.js");
        print("copied: $EXPORT_PATH_DATA/device.js\n");
        copy( $path . $User_Preferences{REL_PATH} . "/device.js", OLD_EXPORT_PATH_3RDPARTY . "device.js");
        print("copied: $EXPORT_PATH_DATA_3RDPARTY/device.js\n");
        copy( $path . $User_Preferences{REL_PATH} . "/platformservices.js", OLD_EXPORT_PATH . "platformservices.js");
        print("copied: $EXPORT_PATH_DATA/platformservices.js\n");
        copy( $path . $User_Preferences{REL_PATH} . "/platformservices.js", EXPORT_PATH_DATA_3RDPARTY . "platformservices.js");
        print("copied: $EXPORT_PATH_DATA_3RDPARTY/platformservices.js\n");
    }
    
}

sub merge()
{
    #create release directory
    mkdir($path . $User_Preferences{REL_PATH});
    #create directory for uncompressed files.
    mkdir($path . $User_Preferences{UNC_PATH});

    #Create empty platformservices.js for CP binary compatability.
    unlink( $path . $User_Preferences{REL_PATH} . "/" . $User_Preferences{COMBINED_JS} );
    open EMPTYFILE, ">", $path . $User_Preferences{UNC_PATH} . "/" . $User_Preferences{COMBINED_JS};
    close EMPTYFILE;


    # Merge all the individual modules into single module js file
    # place them in UNC_PATH directory
    foreach $provider(@providers) {
        #get module directory & open
        $providerjspath = $path . "/" . $provider;
        opendir(DIR, $providerjspath );
        #create result combined file for this module. 
        my $tmpfile =  ">" . $path . $User_Preferences{UNC_PATH} . "/" . $provider . ".js";
        open PROVIDER_JS_FILES, $tmpfile or die $!; 
        while( $file = readdir(DIR) )
        {
            #process only .js files.
            if( rindex($file, ".js") == -1)
            {
                next;
            }
            #copy _data.js files to uncompressed dir.
            if( !(rindex($file, "_data") == -1) )
            {
                copy( $providerjspath . "/" . $file, $path . $User_Preferences{UNC_PATH} . "/" . $file);
                next;
            }
            open (PROVIDERFILE, $providerjspath . "/" . $file);
            while (<PROVIDERFILE>) {
                chomp;
                print PROVIDER_JS_FILES "$_\n";
           }
           close (PROVIDERFILE);
        }
        closedir(DIR);
        close PROVIDER_JS_FILES;
    }

    $jspath = $path . $User_Preferences{UNC_PATH};
    @jsfiles = ();
    opendir(DIR, $jspath) or die "Can't Open Directory $dirname: $!";

    if($doexport == TRUE){
        mkpath(EXPORT_PATH);
        mkpath(EXPORT_PATH_3RDPARTY);
        $EXPORT_PATH_DATA = EXPORT_PATH;
        $EXPORT_PATH_DATA_3RDPARTY = EXPORT_PATH_3RDPARTY;
        
        print("Exporting compressed javascript to wrt shared library...\n");
        print("------------------------------------------------------------\n");
    }

    while( $file = readdir(DIR)  ) {
        #Allow only .js files and skip _data.js files.
        if( rindex($file, ".js") == -1 || !(rindex($file, "_data") == -1) ) { next; }
        if( $compress == TRUE )    {
            system("java -jar $path/../../../tools/packaging/yuicompressor-2.4.2.jar $path$User_Preferences{UNC_PATH}/$file -o $path$User_Preferences{REL_PATH}/$file");
        }
        else{
            copy( $path . $User_Preferences{UNC_PATH} . "/" . $file, $path . $User_Preferences{REL_PATH} . "/" . $file );
        }
        if($doexport == TRUE){
            # EXPORT COMPRESSED JAVASCRIPT.
            copy( $path . $User_Preferences{REL_PATH} . "/" . $file, EXPORT_PATH . $file);
            print("copied: $EXPORT_PATH_DATA$file\n");
            copy( $path . $User_Preferences{REL_PATH} . "/" . $file, EXPORT_PATH_3RDPARTY . $file);
            print("copied: $EXPORT_PATH_DATA_3RDPARTY$file\n");
        }
    }
    closedir(DIR);
    
    if($doexport == TRUE){
        print("\nExporting compressed javascript to nokia shared library...\n");
        print("------------------------------------------------------------\n");
        mkpath(OLD_EXPORT_PATH);
        mkpath(OLD_EXPORT_PATH_3RDPARTY);
        $EXPORT_PATH_DATA = OLD_EXPORT_PATH;
        $EXPORT_PATH_DATA_3RDPARTY = OLD_EXPORT_PATH_3RDPARTY;
        copy( $path . $User_Preferences{REL_PATH} . "/device.js", OLD_EXPORT_PATH . "device.js");
        print("copied: $EXPORT_PATH_DATA/device.js\n");
        copy( $path . $User_Preferences{REL_PATH} . "/device.js", OLD_EXPORT_PATH_3RDPARTY . "device.js");
        print("copied: $EXPORT_PATH_DATA_3RDPARTY/device.js\n");
        copy( $path . $User_Preferences{REL_PATH} . "/platformservices.js", OLD_EXPORT_PATH . "platformservices.js");
        print("copied: $EXPORT_PATH_DATA/platformservices.js\n");
        copy( $path . $User_Preferences{REL_PATH} . "/platformservices.js", OLD_EXPORT_PATH_3RDPARTY . "platformservices.js");
        print("copied: $EXPORT_PATH_DATA_3RDPARTY/platformservices.js\n");
    }
    
}
   
sub cmdWidgets()
{
    cleanup();
    widgets();
} 

sub widgets()
{
    print("generator.pl: creating widgets started....\n");
    $wgtpath = $path . $User_Preferences{WGT_PATH};
    $wgzpath = $path . $User_Preferences{WGZ_PATH};
    mkdir( $path . $User_Preferences{TMP_PATH} );
    mkdir( $wgtpath );
    mkdir( $wgzpath );
    foreach $provider(@providers) {
        $widgetsdir = $path . $User_Preferences{TESTASSET_PATH} . "/". $provider . "/tsrc";
        opendir(WIDGETSDIR, $widgetsdir) || next;
        @widgets = readdir(WIDGETSDIR);
        foreach $widget(@widgets) {
            next if ($widget =~ m/^\./);
            $widgetpath = $widgetsdir . "/" . $widget;
            if( -d $widgetpath )
            {
                $wgzfiles = '';
                $wgtfiles = '';

                #copy testtree into each widget
                copy($path . $User_Preferences{TST_PATH} . "/" . $User_Preferences{TESTTREE_JS}, $widgetpath );
                
                opendir( WIDGETCONTENTS, $widgetpath);
                @widgetcontents = readdir(WIDGETCONTENTS);
                foreach $widgetcontent(@widgetcontents) {
                    next if ($widgetcontent =~ m/^\./);
                    $wgtfiles .= " $widgetcontent";
                    $wgzfiles .= " $widget/$widgetcontent";
                }
                closedir( WIDGETCONTENTS );
            }
            chdir($widgetpath);
            system("$ziptool \"$wgtpath/$widget.zip\" $wgtfiles > \"$path/$User_Preferences{TMP_PATH}/$widget.wgtlog\"  2> \"$path/$User_Preferences{TMP_PATH}/$widget.errwgtlog\"");
            rename "$wgtpath/$widget.zip","$wgtpath/$widget.wgt";
            chdir("$widgetpath/..");
            system("$ziptool \"$wgzpath/$widget.zip\" $wgzfiles > \"$path/$User_Preferences{TMP_PATH}/$widget.wgzlog\"  2> \"$path/$User_Preferences{TMP_PATH}/$widget.errwgzlog\"");
            rename "$wgzpath/$widget.zip","$wgzpath/$widget.wgz";
        }  
        closedir(WIDGETSDIR);
    }
    print("generator.pl: widgets successfully created!!!\n");
}
