/*
 * Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
 *
 * This file is part of Qt Web Runtime.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "wrtcontroller_p.h"
#include "wrtcontroller.h"

#include <qwebframe.h>
#include <qwebview.h>
#include <qgraphicswebview.h>

#include "wrtgraphicswebview.h"
#include "wrtpage.h"

namespace WRT {
    
WrtControllerPrivate::WrtControllerPrivate(QWidget *parent)
    : m_wrtPage(0)
    , m_graphicsWebView(0)
{
    // m_graphicsWebView and m_wrtPage do not need parents, b/c QGraphicsScene
    // will destruct them
    m_graphicsWebView = new WrtGraphicsWebView();
    m_graphicsWebView->setAcceptTouchEvents(true);
    m_wrtPage = new WrtPage();
    m_graphicsWebView->setPage(m_wrtPage);

    if (parent)
        parent->setAttribute(Qt::WA_AcceptTouchEvents, true);
}


WrtControllerPrivate::~WrtControllerPrivate()
{
}

WrtController::WrtController(QWidget *parent)
    : d(new WrtControllerPrivate(parent))
{
}

WrtController::~WrtController()
{
    delete d;
}

WrtPage* WrtController::wrtPage() const
{
    return d->m_wrtPage;
}

QGraphicsWebView* WrtController::graphicsWebView() const
{
    return d->m_graphicsWebView;
}

}

