/*
 * Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
 *
 * This file is part of Qt Web Runtime.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include <QDebug>
#include <QString>
#include <QStringList>

#include "qcontactservice.h"
#include "qjsonizer.h"
#include "qcontactshelpers.h"


namespace ContactsHelpers {

    QVariant createGroupInfoJSON(const QContact *group)
    {
        QVariant retVar = NULL;

        // Insert to map
        QMap<QString,QVariant> valueMap;
        QMap<QString,QVariant> tmpMap;

        tmpMap.insert("groupId",QVariant(group->id().localId()));
        tmpMap.insert("groupName",QVariant(group->displayLabel()));

        valueMap.insert("group",QVariant(tmpMap));

        QList<QVariant> memberList;
        QContactManager cm;
        //QList<QUniqueId> tmpList(group->members());
        QContactId grpId;
        grpId.setLocalId(group->id().localId());
        QList<QContactRelationship> tmpList = cm.relationships(QContactRelationship::HasMember, grpId);

        foreach (const QContactRelationship& currRel, tmpList) {
            memberList.append(currRel.second().localId());
        }

        valueMap.insert("contents",QVariant(memberList));

        retVar = valueMap;

        return retVar;
    }

    void setContactFields(QContact * con,
            const QMap<QString, QVariant> & jsonObject, QMap<QString, QVariant> &result)
    {
        QMapIterator<QString, QVariant> iterator(jsonObject);
        while (iterator.hasNext()) {
            iterator.next();
            setContactField(con, iterator.key(), iterator.value(), result);
            if(result[KErrorCode] != 0)
            {
                break;
            }
        }
    }

    void setContactField(QContact* con, QString key, QVariant data, QMap<QString, QVariant> &result)
    {
        

        if (key == "name") {
            QContactName name = con->detail(QContactName::DefinitionName);
            if(data.toMap().value("last").type() == QVariant::String) {
                name.setLastName(data.toMap().value("last").toString());
            }
            if(data.toMap().value("first").type() == QVariant::String) {
                name.setFirstName(data.toMap().value("first").toString());
            }
            if(data.toMap().value("middle").type() == QVariant::String) {
                name.setMiddleName(data.toMap().value("middle").toString());
            }
            if(data.toMap().value("suffix").type() == QVariant::String) {
                name.setSuffix(data.toMap().value("suffix").toString());
            }
            if(data.toMap().value("prefix").type() == QVariant::String) {
                name.setPrefix(data.toMap().value("prefix").toString());
            }

            con->saveDetail(&name);
        } else if (key.startsWith("tel")) {
            
            QStringList phoneNumberList;
            
            if (data.toMap().value("land").type() == QVariant::String) {
                phoneNumberList.append(data.toMap().value("land").toString());
            } 
            else if (data.toMap().value("land").type() == QVariant::List) {
                phoneNumberList = data.toMap().value("land").toStringList();
            }
            for (int i=0; i<phoneNumberList.length(); i++) {
                QContactPhoneNumber telLand; 
                telLand.setSubTypes(QContactPhoneNumber::SubTypeLandline);
                telLand.setNumber(phoneNumberList.at(i));
                bool conSaved = con->saveDetail(&telLand);
            }
            phoneNumberList.clear();

            if (data.toMap().value("mobile").type() == QVariant::String) {
                phoneNumberList.append(data.toMap().value("mobile").toString());
            } else if (data.toMap().value("mobile").type() == QVariant::List) {
                phoneNumberList = data.toMap().value("mobile").toStringList();
            }
            for (int i=0; i<phoneNumberList.length(); i++) {
                QContactPhoneNumber telMobile; 
                telMobile.setSubTypes(QContactPhoneNumber::SubTypeMobile);
                telMobile.setNumber(phoneNumberList.at(i));
                con->saveDetail(&telMobile);
            }
            phoneNumberList.clear();
            
            if (data.toMap().value("fax").type() == QVariant::String) {
                phoneNumberList.append(data.toMap().value("fax").toString());
            } else if (data.toMap().value("fax").type() == QVariant::List) {
                phoneNumberList = data.toMap().value("fax").toStringList();
            }
            for (int i=0; i<phoneNumberList.length(); i++) {
                QContactPhoneNumber telFax; 
                telFax.setSubTypes(QContactPhoneNumber::SubTypeModem);
                telFax.setNumber(phoneNumberList.at(i));
                con->saveDetail(&telFax);
            }            
            phoneNumberList.clear();
            
            if (data.toMap().value("voip").type() == QVariant::String) {
                phoneNumberList.append(data.toMap().value("voip").toString());
            } else if (data.toMap().value("voip").type() == QVariant::List) {
                phoneNumberList = data.toMap().value("voip").toStringList();
            }
            for (int i=0; i<phoneNumberList.length(); i++) {
                QContactPhoneNumber telVoice; 
                telVoice.setSubTypes(QContactPhoneNumber::SubTypeVoice);
                telVoice.setNumber(phoneNumberList.at(i));
                con->saveDetail(&telVoice);
            }
            phoneNumberList.clear();
            
            if (data.toMap().value("video").type() == QVariant::String) {
                phoneNumberList.append(data.toMap().value("video").toString());
            } else if (data.toMap().value("video").type() == QVariant::List) {
                phoneNumberList = data.toMap().value("video").toStringList();
            }
            for (int i=0; i<phoneNumberList.length(); i++) {
                QContactPhoneNumber telVideo; 
                telVideo.setSubTypes(QContactPhoneNumber::SubTypeVideo);
                telVideo.setNumber(phoneNumberList.at(i));
                con->saveDetail(&telVideo);
            }
            phoneNumberList.clear();  
            
            //work
            if (data.toMap().value("work").toMap().value("land").type() == QVariant::String) {
                phoneNumberList.append(data.toMap().value("work").toMap().value("land").toString());
            } else if (data.toMap().value("work").toMap().value("land").type() == QVariant::List) {
                phoneNumberList = data.toMap().value("work").toMap().value("land").toStringList();
            }
            for (int i=0; i<phoneNumberList.length(); i++) {
                QContactPhoneNumber telLand2; 
                telLand2.setContexts(QContactPhoneNumber::ContextWork);
                telLand2.setSubTypes(QContactPhoneNumber::SubTypeLandline);
                telLand2.setNumber(phoneNumberList.at(i));
                con->saveDetail(&telLand2);
            }
            phoneNumberList.clear();

            if (data.toMap().value("work").toMap().value("mobile").type() == QVariant::String) {
                phoneNumberList.append(data.toMap().value("work").toMap().value("mobile").toString());
            } else if (data.toMap().value("work").toMap().value("mobile").type() == QVariant::List) {
                phoneNumberList = data.toMap().value("work").toMap().value("mobile").toStringList();
            }
            for (int i=0; i<phoneNumberList.length(); i++) {
                QContactPhoneNumber telMobile2; 
                telMobile2.setContexts(QContactPhoneNumber::ContextWork);
                telMobile2.setSubTypes(QContactPhoneNumber::SubTypeMobile);
                telMobile2.setNumber(phoneNumberList.at(i));
                con->saveDetail(&telMobile2);
            }
            phoneNumberList.clear();
            
            if (data.toMap().value("work").toMap().value("fax").type() == QVariant::String) {
                phoneNumberList.append(data.toMap().value("work").toMap().value("fax").toString());
            } else if (data.toMap().value("work").toMap().value("fax").type() == QVariant::List) {
                phoneNumberList = data.toMap().value("work").toMap().value("fax").toStringList();
            }
            for (int i=0; i<phoneNumberList.length(); i++) {
                QContactPhoneNumber telFax2; 
                telFax2.setContexts(QContactPhoneNumber::ContextWork);
                telFax2.setSubTypes(QContactPhoneNumber::SubTypeModem);
                telFax2.setNumber(phoneNumberList.at(i));
                con->saveDetail(&telFax2);
            }            
            phoneNumberList.clear();
            
            if (data.toMap().value("work").toMap().value("voip").type() == QVariant::String) {
                phoneNumberList.append(data.toMap().value("work").toMap().value("voip").toString());
            } else if (data.toMap().value("work").toMap().value("voip").type() == QVariant::List) {
                phoneNumberList = data.toMap().value("work").toMap().value("voip").toStringList();
            }
            for (int i=0; i<phoneNumberList.length(); i++) {
                QContactPhoneNumber telVoice2; 
                telVoice2.setContexts(QContactPhoneNumber::ContextWork);
                telVoice2.setSubTypes(QContactPhoneNumber::SubTypeVoice);
                telVoice2.setNumber(phoneNumberList.at(i));
                con->saveDetail(&telVoice2);
            }
            phoneNumberList.clear();
            
            if (data.toMap().value("work").toMap().value("video").type() == QVariant::String) {
                phoneNumberList.append(data.toMap().value("work").toMap().value("video").toString());
            } else if (data.toMap().value("work").toMap().value("video").type() == QVariant::List) {
                phoneNumberList = data.toMap().value("work").toMap().value("video").toStringList();
            }
            for (int i=0; i<phoneNumberList.length(); i++) {
                QContactPhoneNumber telVideo2; 
                telVideo2.setContexts(QContactPhoneNumber::ContextWork);
                telVideo2.setSubTypes(QContactPhoneNumber::SubTypeVideo);
                telVideo2.setNumber(phoneNumberList.at(i));
                con->saveDetail(&telVideo2);
            }
            phoneNumberList.clear();             
            
            //home
            if (data.toMap().value("home").toMap().value("land").type() == QVariant::String) {
                phoneNumberList.append(data.toMap().value("home").toMap().value("land").toString());
            } else if (data.toMap().value("home").toMap().value("land").type() == QVariant::List) {
                phoneNumberList = data.toMap().value("home").toMap().value("land").toStringList();
            }
            for (int i=0; i<phoneNumberList.length(); i++) {
                QContactPhoneNumber telLand3; 
                telLand3.setContexts(QContactPhoneNumber::ContextHome);
                telLand3.setSubTypes(QContactPhoneNumber::SubTypeLandline);
                telLand3.setNumber(phoneNumberList.at(i));
                con->saveDetail(&telLand3);
            }
            phoneNumberList.clear();

            if (data.toMap().value("home").toMap().value("mobile").type() == QVariant::String) {
                phoneNumberList.append(data.toMap().value("home").toMap().value("mobile").toString());
            } else if (data.toMap().value("home").toMap().value("mobile").type() == QVariant::List) {
                phoneNumberList = data.toMap().value("home").toMap().value("mobile").toStringList();
            }
            for (int i=0; i<phoneNumberList.length(); i++) {
                QContactPhoneNumber telMobile3; 
                telMobile3.setContexts(QContactPhoneNumber::ContextHome);
                telMobile3.setSubTypes(QContactPhoneNumber::SubTypeMobile);
                telMobile3.setNumber(phoneNumberList.at(i));
                con->saveDetail(&telMobile3);
            }
            phoneNumberList.clear();
            
            if (data.toMap().value("home").toMap().value("fax").type() == QVariant::String) {
                phoneNumberList.append(data.toMap().value("home").toMap().value("fax").toString());
            } else if (data.toMap().value("home").toMap().value("fax").type() == QVariant::List) {
                phoneNumberList = data.toMap().value("home").toMap().value("fax").toStringList();
            }
            for (int i=0; i<phoneNumberList.length(); i++) {
                QContactPhoneNumber telFax3; 
                telFax3.setContexts(QContactPhoneNumber::ContextHome);
                telFax3.setSubTypes(QContactPhoneNumber::SubTypeModem);
                telFax3.setNumber(phoneNumberList.at(i));
                con->saveDetail(&telFax3);
            }            
            phoneNumberList.clear();
            
            if (data.toMap().value("home").toMap().value("voip").type() == QVariant::String) {
                phoneNumberList.append(data.toMap().value("home").toMap().value("voip").toString());
            } else if (data.toMap().value("home").toMap().value("voip").type() == QVariant::List) {
                phoneNumberList = data.toMap().value("home").toMap().value("voip").toStringList();
            }
            for (int i=0; i<phoneNumberList.length(); i++) {
                QContactPhoneNumber telVoice3; 
                telVoice3.setContexts(QContactPhoneNumber::ContextHome);
                telVoice3.setSubTypes(QContactPhoneNumber::SubTypeVoice);
                telVoice3.setNumber(phoneNumberList.at(i));
                con->saveDetail(&telVoice3);
            }
            phoneNumberList.clear();
            
            if (data.toMap().value("home").toMap().value("video").type() == QVariant::String) {
                phoneNumberList.append(data.toMap().value("home").toMap().value("video").toString());
            } else if (data.toMap().value("home").toMap().value("video").type() == QVariant::List) {
                phoneNumberList = data.toMap().value("home").toMap().value("video").toStringList();
            }
            for (int i=0; i<phoneNumberList.length(); i++) {
                QContactPhoneNumber telVideo3; 
                telVideo3.setContexts(QContactPhoneNumber::ContextHome);
                telVideo3.setSubTypes(QContactPhoneNumber::SubTypeVideo);
                telVideo3.setNumber(phoneNumberList.at(i));
                con->saveDetail(&telVideo3);
            }
            phoneNumberList.clear();         
            
        } else if (key.startsWith("address")) {
            QContactAddress address = con->detail(QContactAddress::DefinitionName);
            if (data.toMap().value("street").type() == QVariant::String) {
                address.setStreet(data.toMap().value("street").toString());
            }
            if (data.toMap().value("local").type() == QVariant::String) {
                address.setLocality(data.toMap().value("local").toString());
            }
            if (data.toMap().value("region").type() == QVariant::String) {
                address.setRegion(data.toMap().value("region").toString());
            }
            if (data.toMap().value("code").type() == QVariant::String) {
                address.setPostcode(data.toMap().value("code").toString());
            }
            if (data.toMap().value("country").type() == QVariant::String) {
                address.setCountry(data.toMap().value("country").toString());
            }
            
            con->saveDetail(&address);
            QStringList emailList;
            if (data.toMap().value("email").type() == QVariant::String) {
                emailList.append(data.toMap().value("email").toString());
            } else if (data.toMap().value("email").type() == QVariant::List) {
                emailList = data.toMap().value("email").toStringList();
            }
            for (int i=0; i<emailList.length(); i++) {
                QContactEmailAddress email; 
                email.setEmailAddress(emailList.at(i));
                con->saveDetail(&email);
            }
            emailList.clear();
            
            if (data.toMap().value("uri").type() == QVariant::String) {
                QContactUrl url = con->detail(QContactUrl::DefinitionName);
                url.setUrl(data.toMap().value("uri").toString());
                con->saveDetail(&url);
            }
    
            //Work            
            QContactAddress address2 = con->detail(QContactAddress::DefinitionName);
            address2.setContexts(QContactAddress::ContextWork);
            if (data.toMap().value("work").toMap().value("street").type() == QVariant::String) {
                address2.setStreet(data.toMap().value("work").toMap().value("street").toString());
            }
            if (data.toMap().value("work").toMap().value("local").type() == QVariant::String) {
                address2.setLocality(data.toMap().value("work").toMap().value("local").toString());
            }
            if (data.toMap().value("work").toMap().value("region").type() == QVariant::String) {
                address2.setRegion(data.toMap().value("work").toMap().value("region").toString());
            }
            if (data.toMap().value("work").toMap().value("code").type() == QVariant::String) {
                address2.setPostcode(data.toMap().value("work").toMap().value("code").toString());
            }
            if (data.toMap().value("work").toMap().value("country").type() == QVariant::String) {
                address2.setCountry(data.toMap().value("work").toMap().value("country").toString());
            }
            
            con->saveDetail(&address2);
            
            if (data.toMap().value("work").toMap().value("email").type() == QVariant::String) {
                emailList.append(data.toMap().value("work").toMap().value("email").toString());
            } else if (data.toMap().value("work").toMap().value("email").type() == QVariant::List) {
                emailList = data.toMap().value("work").toMap().value("email").toStringList();
            }
            for (int i=0; i<emailList.length(); i++) {
                QContactEmailAddress email2; 
                email2.setContexts(QContactEmailAddress::ContextWork);
                email2.setEmailAddress(emailList.at(i));
                con->saveDetail(&email2);
            }
            emailList.clear();

            if (data.toMap().value("work").toMap().value("uri").type() == QVariant::String) {
                QContactUrl url2 = con->detail(QContactUrl::DefinitionName);
                url2.setContexts(QContactUrl::ContextWork);
                url2.setUrl(data.toMap().value("work").toMap().value("uri").toString());
                con->saveDetail(&url2);
            }

            QContactAddress address3 = con->detail(QContactAddress::DefinitionName);
            address3.setContexts(QContactAddress::ContextHome);
            if (data.toMap().value("home").toMap().value("street").type() == QVariant::String) {
                address3.setStreet(data.toMap().value("home").toMap().value("street").toString());
            }
            if (data.toMap().value("home").toMap().value("local").type() == QVariant::String) {
                address3.setLocality(data.toMap().value("home").toMap().value("local").toString());
            }
            if (data.toMap().value("home").toMap().value("region").type() == QVariant::String) {
                address3.setRegion(data.toMap().value("home").toMap().value("region").toString());
            }
            if (data.toMap().value("home").toMap().value("code").type() == QVariant::String) {
                address3.setPostcode(data.toMap().value("home").toMap().value("code").toString());
            }
            if (data.toMap().value("home").toMap().value("country").type() == QVariant::String) {
                address3.setCountry(data.toMap().value("home").toMap().value("country").toString());
            }
            
            con->saveDetail(&address3);
            
            if (data.toMap().value("home").toMap().value("email").type() == QVariant::String) {
                emailList.append(data.toMap().value("home").toMap().value("email").toString());
            } else if (data.toMap().value("home").toMap().value("email").type() == QVariant::List) {
                emailList = data.toMap().value("home").toMap().value("email").toStringList();
            }
            for (int i=0; i<emailList.length(); i++) {
                QContactEmailAddress email3; 
                email3.setContexts(QContactEmailAddress::ContextHome);
                email3.setEmailAddress(emailList.at(i));
                con->saveDetail(&email3);
            }
            emailList.clear();            

            if (data.toMap().value("home").toMap().value("uri").type() == QVariant::String) {
                QContactUrl url3 = con->detail(QContactUrl::DefinitionName);
                url3.setContexts(QContactUrl::ContextHome);
                url3.setUrl(data.toMap().value("home").toMap().value("uri").toString());
                con->saveDetail(&url3);
            }
    
        } else if (key.startsWith("company")) {
            QContactOrganization organization = con->detail(QContactOrganization::DefinitionName);

            organization.setName(data.toMap().value("name").toString());
            organization.setTitle(data.toMap().value("title").toString());

            con->saveDetail(&organization);
        } else if (key == "id") {
            // Id present if the use case is updateContact
            bool ok;
            QContactLocalId id = data.toUInt(&ok);
            if (ok) {
                QContactManager* tempCm = new QContactManager();
                QContact contact = tempCm->contact(id);
                if (!contact.isEmpty()) {
                    QContactId cId;
                    cId.setLocalId(data.toUInt());
//                    cId.setManagerUri("qtcontacts:symbian:");
                    cId.setManagerUri(cId.managerUri());
                    con->setId(cId);
                }
            }
        } else if (key == "notes") {
            QContactNote notes = con->detail(QContactNote::DefinitionName);
            notes.setNote(data.toString());
            con->saveDetail(&notes);
        } else if (key == "anniversary") {
            QContactAnniversary anniversary = con->detail(QContactAnniversary::DefinitionName);
            QDateTime dateTime(reinterpret_cast<QDateTime&>(data));
            anniversary.setOriginalDate(dateTime.date());
            con->saveDetail(&anniversary);
        } else if (key == "birthday") {
            QContactBirthday birthday = con->detail(QContactBirthday::DefinitionName);
            QDateTime dateTime(reinterpret_cast<QDateTime&>(data));
            birthday.setDate(dateTime.date());
            con->saveDetail(&birthday);
        } else if (key == "nickname") {
            QContactNickname nickname = con->detail(QContactNickname::DefinitionName);
            nickname.setNickname(data.toString());
            con->saveDetail(&nickname);
        } else if (key == "photo") {
            qDebug() <<"imageuri" <<data.toString();
            if(data.isNull())
            {
                return;
            }
            if(data.type() != QVariant::String)
            {
                qDebug() <<"not qstring";
                result[KErrorCode] = CJSEStatusCodes::INVALID_ARG_ERR;
                result[KErrorMessage] = "Invalid photo parameter";
                return;
            }
            
            QString imagepath(data.toString());
            qDebug() <<"imagepath is assigned";
            if (imagepath.contains(FILE_PROTOCOL))
            {
                qDebug() << "correct file protocol";
                imagepath = imagepath.right(imagepath.length() - KFileProtocolLen);
                qDebug() <<"imagepath"<<imagepath;
                QString onlypath = imagepath;
                int pathlen = onlypath.lastIndexOf("/");
                onlypath.truncate(pathlen+1);
                qDebug() <<"after truncate" <<imagepath <<pathlen;
                //take only the filename
                
                QString onlyname = imagepath.right(((imagepath.length())-(pathlen+1)));
                
                qDebug()<<"only filename"<<onlyname;
                
                QFile file;
                QDir::setCurrent(onlypath);
                file.setFileName(onlyname);
                if (file.exists())
                {
                    qDebug() <<"file exists, so assigning";
                    QUrl imageurl(data.toString());
                    QContactAvatar avatar = con->detail(QContactAvatar::DefinitionName);
                    avatar.setImageUrl(imageurl);
                    con->saveDetail(&avatar);    
                }
                else
                {
                    result[KErrorCode] = CJSEStatusCodes::DATA_NOT_FOUND_ERR;
                    result[KErrorMessage] = "The photo input is invalid";
                    return;
                }
            }
            else
            {
                result[KErrorCode] = CJSEStatusCodes::INVALID_ARG_ERR;
                result[KErrorMessage] = "Invalid photo input";
                return;
            } 
        } else if (key == "displayName") {
            QContactName displayLabel = con->detail(QContactName::DefinitionName);
            displayLabel.setCustomLabel(QString(data.toString()));
            con->saveDetail(&displayLabel);
        } else if (key == "xspid") {
            // No point to set xspids?
        }
    }

// ContactsHelpers Namespace ends
}

