/*
 * Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
 *
 * This file is part of Qt Web Runtime.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef GETCONTACTIDSTASK_H_
#define GETCONTACTIDSTASK_H_

#include <QObject>

// Qt Mobility header
#include <qtcontacts.h>

using namespace QTM_NAMESPACE;

/**
 * An async task for invoking Contacts::getContactsIds().
 */
class GetContactIdsTask: public QObject {

    Q_OBJECT

public:
    GetContactIdsTask(QContactManager* pcm, QString match, qint32 sortOrder, qint32 trId);
    virtual ~GetContactIdsTask();
    QList<QContactLocalId> doGetContactIds();

    virtual void run();
public slots:
    void start();

Q_SIGNALS:
    /**
     * Signals a contactIds with status.
     * @param a list that contains all contact ids.
     */
    void contactIds(GetContactIdsTask* aTask, QList<QContactLocalId>, qint32 trId, qint32 status);

private:
    QContactManager* m_pcm;
    QString m_match;
    qint32 m_sortOrder;
    qint32 m_trId;
};

#endif /* GETCONTACTIDSTASK_H_ */
