/*
 * Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
 *
 * This file is part of Qt Web Runtime.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include <QList>
#include <QtDebug>
#include "qgetgroupstask.h"
#include "qcontactshelpers.h"

GetGroupsTask::GetGroupsTask(QContactManager* pcm, qint32 trId) :
   m_pcm(pcm), m_trId(trId) {
}

GetGroupsTask::~GetGroupsTask()
{
}

void GetGroupsTask::run()
{
    // Getting groups via QtContacts
    QList<QContact> groupList;
    QContactDetailFilter groupFilter;
    groupFilter.setDetailDefinitionName(QContactType::DefinitionName, QContactType::TypeGroup);
    groupFilter.setValue(QString(QLatin1String(QContactType::TypeGroup)));

    QContactFetchRequest fetch;

    fetch.setFilter(groupFilter);
    fetch.setManager(m_pcm);


    if (!fetch.start()) {
        qDebug() << "Failed to start group fetch request!" << fetch.error();
        // Start failed
        emit groupInfos(this, groupList, m_trId,
                CJSEStatusCodes::DATA_NOT_FOUND_ERR);
    } else {
        qDebug() << "Get group request started...";
        fetch.waitForFinished();
        qDebug() << "Get group request end";

        groupList = fetch.contacts();
        qDebug() << "Number of groups: " <<  groupList.count();

        if (groupList.isEmpty()) {
            emit groupInfos(this, groupList, m_trId, CJSEStatusCodes::DATA_NOT_FOUND_ERR);
        } else {
            emit groupInfos(this, groupList, m_trId, CJSEStatusCodes::SUCCESS);
        }
    }
}


void GetGroupsTask::start()
{
    run();
    delete this;
}

