/*
 * Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
 *
 * This file is part of Qt Web Runtime.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "qjsonizer.h"

#include <QStringList>
#include <QMap>
#include <QtDebug>

JSonizer::JSonizer()
{
    m_contactMap.clear();
}

JSonizer::~JSonizer() {
}

QVariant JSonizer::contact2json(const QContact *contact)
{
    QVariant var = NULL;

    nameFields2json(contact);
    telFields2json(contact);
    addressFields2json(contact);
    companyFields2json(contact);
    miscFields2json(contact);
    var = m_contactMap;

    return var;
}

void JSonizer::nameFields2json(const QContact * contact)
{
    QContactName name = contact->detail(QContactName::DefinitionName);
    // Insert to Contact map
    QMap<QString,QVariant> valueMap;
    if(!((name.lastName()).isEmpty()))
        valueMap.insert("last",QVariant(name.lastName()));
    if(!((name.firstName()).isEmpty()))
        valueMap.insert("first",QVariant(name.firstName()));
    if(!((name.middleName()).isEmpty()))
        valueMap.insert("middle",QVariant(name.middleName()));
    if(!((name.prefix()).isEmpty()))
        valueMap.insert("prefix",QVariant(name.prefix()));
    if(!((name.suffix()).isEmpty()))
        valueMap.insert("suffix",QVariant(name.suffix()));
    if(valueMap.size() > 0)
        m_contactMap.insert(QString("name"),QVariant(valueMap));
}


void JSonizer::telFields2json(const QContact * contact)
{
    //TODO rework it if possible
    QMap<QString,QVariant> valueMap;
    QMap<QString,QVariant> tmpHomeMap;
    QMap<QString,QVariant> tmpWorkMap;
    QList<QContactDetail> generalPhoneNumbers = contact->details(QContactPhoneNumber::DefinitionName);

     QContactPhoneNumber tel = contact->detail(QContactPhoneNumber::DefinitionName);
    qDebug() << "phonenumber in jsoniser: " << tel.number();
    
    for (int k=0; k < generalPhoneNumbers.size(); k++) {    
        QContactPhoneNumber phoneDetail = generalPhoneNumbers.at(k);
        
        if (!phoneDetail.contexts().size()) {
            if (phoneDetail.subTypes().at(0) == QContactPhoneNumber::SubTypeLandline) {
                if(valueMap.contains("land")) {
                    QStringList phoneList;
                    if(valueMap.value("land").type() == QVariant::String) {
                        phoneList.append(valueMap.value("land").toString());
                        phoneList.append(phoneDetail.number());
                        valueMap.insert("land",QVariant(phoneDetail.number())); 
                    } else if(valueMap.value("land").type() == QVariant::List) {
                        phoneList = valueMap.value("land").toStringList();
                        phoneList.append(phoneDetail.number());
                        valueMap.insert("land", QVariant(phoneList));
                    }
                    valueMap.insert("land", QVariant(phoneList));
                } else {
                    valueMap.insert("land",QVariant(phoneDetail.number()));
                }
            } 
            
            
            
            else if (phoneDetail.subTypes().at(0) == QContactPhoneNumber::SubTypeMobile) {
                if(valueMap.contains("mobile")) {
                    QStringList phoneList;
                    if(valueMap.value("mobile").type() == QVariant::String) {
                        phoneList.append(valueMap.value("mobile").toString());
                        phoneList.append(phoneDetail.number());
                    } else if(valueMap.value("mobile").type() == QVariant::List) {
                        phoneList = valueMap.value("mobile").toStringList();
                        phoneList.append(phoneDetail.number());
                    }
                    valueMap.insert("mobile", QVariant(phoneList));
                } else {
                    valueMap.insert("mobile",QVariant(phoneDetail.number()));
                }
            } else if (phoneDetail.subTypes().at(0) == QContactPhoneNumber::SubTypeVideo) {
                if(valueMap.contains("video")) {
                    QStringList phoneList;
                    if(valueMap.value("video").type() == QVariant::String) {
                        phoneList.append(valueMap.value("video").toString());
                        phoneList.append(phoneDetail.number());
                    } else if(valueMap.value("video").type() == QVariant::List) {
                        phoneList = valueMap.value("video").toStringList();
                        phoneList.append(phoneDetail.number());
                    }
                    valueMap.insert("video", QVariant(phoneList));
                } else {
                    valueMap.insert("video",QVariant(phoneDetail.number()));
                }            
            } else if (phoneDetail.subTypes().at(0) == QContactPhoneNumber::SubTypeModem) {
                if(valueMap.contains("fax")) {
                    QStringList phoneList;
                    if(valueMap.value("fax").type() == QVariant::String) {
                        phoneList.append(valueMap.value("fax").toString());
                        phoneList.append(phoneDetail.number());
                    } else if(valueMap.value("fax").type() == QVariant::List) {
                        phoneList = valueMap.value("fax").toStringList();
                        phoneList.append(phoneDetail.number());
                    }
                    valueMap.insert("fax", QVariant(phoneList));
                } else {
                    valueMap.insert("fax",QVariant(phoneDetail.number()));
                }               
            } else if (phoneDetail.subTypes().at(0) == QContactPhoneNumber::SubTypeVoice) {
                if(valueMap.contains("voip")) {
                    QStringList phoneList;
                    if(valueMap.value("voip").type() == QVariant::String) {
                        phoneList.append(valueMap.value("voip").toString());
                        phoneList.append(phoneDetail.number());
                    } else if(valueMap.value("voip").type() == QVariant::List) {
                        phoneList = valueMap.value("voip").toStringList();
                        phoneList.append(phoneDetail.number());
                    }
                    valueMap.insert("voip", QVariant(phoneList));
                } else {
                    valueMap.insert("voip",QVariant(phoneDetail.number()));
                }               
            }
        } else {
            qDebug() <<"have some contexts" << phoneDetail.contexts().size();
            
            if (phoneDetail.contexts().at(0) == QContactPhoneNumber::ContextHome) {
                qDebug() <<"context home";
                if (phoneDetail.subTypes().at(0) == QContactPhoneNumber::SubTypeLandline) {
                    qDebug() <<"subtype landline";
                    if(tmpHomeMap.contains("land")) {
                        QStringList phoneList;
                        if(tmpHomeMap.value("land").type() == QVariant::String) {
                            phoneList.append(tmpHomeMap.value("land").toString());
                            phoneList.append(phoneDetail.number());
                        } else if(tmpHomeMap.value("land").type() == QVariant::List) {
                            phoneList = tmpHomeMap.value("land").toStringList();
                            phoneList.append(phoneDetail.number());
                        }
                        tmpHomeMap.insert("land", QVariant(phoneList));
                    } else {
                    tmpHomeMap.insert("land",QVariant(phoneDetail.number()));
                    }                
                } else if (phoneDetail.subTypes().at(0) == QContactPhoneNumber::SubTypeMobile){
                    qDebug() <<"mobile subtype";
                    if(tmpHomeMap.contains("mobile")) {
                        QStringList phoneList;
                        if(tmpHomeMap.value("mobile").type() == QVariant::String) {
                            phoneList.append(tmpHomeMap.value("mobile").toString());
                            phoneList.append(phoneDetail.number());
                        } else if(tmpHomeMap.value("mobile").type() == QVariant::List) {
                            phoneList = tmpHomeMap.value("mobile").toStringList();
                            phoneList.append(phoneDetail.number());
                        }
                        tmpHomeMap.insert("mobile", QVariant(phoneList));
                    } else {
                    tmpHomeMap.insert("mobile",QVariant(phoneDetail.number()));
                    }                
                } else if (phoneDetail.subTypes().at(0) == QContactPhoneNumber::SubTypeVideo){
                     qDebug() <<"VIDEO SUBTYPE"; 
                    if(tmpHomeMap.contains("video")) {
                        QStringList phoneList;
                        if(tmpHomeMap.value("video").type() == QVariant::String) {
                            phoneList.append(tmpHomeMap.value("video").toString());
                            phoneList.append(phoneDetail.number());
                        } else if(tmpHomeMap.value("video").type() == QVariant::List) {
                            phoneList = tmpHomeMap.value("video").toStringList();
                            phoneList.append(phoneDetail.number());
                        }
                        tmpHomeMap.insert("video", QVariant(phoneList));
                    } else {
                    tmpHomeMap.insert("video",QVariant(phoneDetail.number()));
                    }                
                } else if (phoneDetail.subTypes().at(0) == QContactPhoneNumber::SubTypeModem){
                    qDebug() <<"fax subtype";
                    if(tmpHomeMap.contains("fax")) {
                        QStringList phoneList;
                        if(tmpHomeMap.value("fax").type() == QVariant::String) {
                            phoneList.append(tmpHomeMap.value("fax").toString());
                            phoneList.append(phoneDetail.number());
                        } else if(tmpHomeMap.value("fax").type() == QVariant::List) {
                            phoneList = tmpHomeMap.value("fax").toStringList();
                            phoneList.append(phoneDetail.number());
                        }
                        tmpHomeMap.insert("fax", QVariant(phoneList));
                    } else {
                    tmpHomeMap.insert("fax",QVariant(phoneDetail.number()));
                    }                
                } else if (phoneDetail.subTypes().at(0) == QContactPhoneNumber::SubTypeVoice){
                    qDebug() <<"voip ST";
                    if(tmpHomeMap.contains("voip")) {
                        QStringList phoneList;
                        if(tmpHomeMap.value("voip").type() == QVariant::String) {
                            phoneList.append(tmpHomeMap.value("voip").toString());
                            phoneList.append(phoneDetail.number());
                        } else if(tmpHomeMap.value("voip").type() == QVariant::List) {
                            phoneList = tmpHomeMap.value("voip").toStringList();
                            phoneList.append(phoneDetail.number());
                        }
                        tmpHomeMap.insert("voip", QVariant(phoneList));
                    } else {
                    tmpHomeMap.insert("voip",QVariant(phoneDetail.number()));
                    }                
                }            
            } else if (phoneDetail.contexts().at(0) == QContactPhoneNumber::ContextWork) {
                qDebug() <<"context work";
                if (phoneDetail.subTypes().at(0) == QContactPhoneNumber::SubTypeLandline) {
                    if(tmpWorkMap.contains("land")) {
                        QStringList phoneList;
                        if(tmpWorkMap.value("land").type() == QVariant::String) {
                            phoneList.append(tmpWorkMap.value("land").toString());
                            phoneList.append(phoneDetail.number());
                        } else if(tmpWorkMap.value("land").type() == QVariant::List) {
                            phoneList = tmpWorkMap.value("land").toStringList();
                            phoneList.append(phoneDetail.number());
                        }
                        tmpWorkMap.insert("land", QVariant(phoneList));
                    } else {
                    tmpWorkMap.insert("land",QVariant(phoneDetail.number()));
                    }                
                } else if (phoneDetail.subTypes().at(0) == QContactPhoneNumber::SubTypeMobile){
                    if(tmpWorkMap.contains("mobile")) {
                        QStringList phoneList;
                        if(tmpWorkMap.value("mobile").type() == QVariant::String) {
                            phoneList.append(tmpWorkMap.value("mobile").toString());
                            phoneList.append(phoneDetail.number());
                        } else if(tmpWorkMap.value("mobile").type() == QVariant::List) {
                            phoneList = tmpWorkMap.value("mobile").toStringList();
                            phoneList.append(phoneDetail.number());
                        }
                        tmpWorkMap.insert("mobile", QVariant(phoneList));
                    } else {
                    tmpWorkMap.insert("mobile",QVariant(phoneDetail.number()));
                    }                
                } else if (phoneDetail.subTypes().at(0) == QContactPhoneNumber::SubTypeVideo){
                    if(tmpWorkMap.contains("video")) {
                        QStringList phoneList;
                        if(tmpWorkMap.value("video").type() == QVariant::String) {
                            phoneList.append(tmpWorkMap.value("video").toString());
                            phoneList.append(phoneDetail.number());
                        } else if(tmpWorkMap.value("video").type() == QVariant::List) {
                            phoneList = tmpWorkMap.value("video").toStringList();
                            phoneList.append(phoneDetail.number());
                        }
                        tmpWorkMap.insert("video", QVariant(phoneList));
                    } else {
                    tmpWorkMap.insert("video",QVariant(phoneDetail.number()));
                    }                
                } else if (phoneDetail.subTypes().at(0) == QContactPhoneNumber::SubTypeModem){
                    if(tmpWorkMap.contains("fax")) {
                        QStringList phoneList;
                        if(tmpWorkMap.value("fax").type() == QVariant::String) {
                            phoneList.append(tmpWorkMap.value("fax").toString());
                            phoneList.append(phoneDetail.number());
                        } else if(tmpWorkMap.value("fax").type() == QVariant::List) {
                            phoneList = tmpWorkMap.value("fax").toStringList();
                            phoneList.append(phoneDetail.number());
                        }
                        tmpWorkMap.insert("fax", QVariant(phoneList));
                    } else {
                    tmpWorkMap.insert("fax",QVariant(phoneDetail.number()));
                    }                
                } else if (phoneDetail.subTypes().at(0) == QContactPhoneNumber::SubTypeVoice){
                    if(tmpWorkMap.contains("voip")) {
                        QStringList phoneList;
                        if(tmpWorkMap.value("voip").type() == QVariant::String) {
                            phoneList.append(tmpWorkMap.value("voip").toString());
                            phoneList.append(phoneDetail.number());
                        } else if(tmpWorkMap.value("voip").type() == QVariant::List) {
                            phoneList = tmpWorkMap.value("voip").toStringList();
                            phoneList.append(phoneDetail.number());
                        }
                        tmpWorkMap.insert("voip", QVariant(phoneList));
                    } else {
                    tmpWorkMap.insert("voip",QVariant(phoneDetail.number()));
                    }                
                }
            }
        }
    }
    if(tmpHomeMap.size() > 0)
        valueMap.insert("home",QVariant(tmpHomeMap));
    if(tmpWorkMap.size() > 0)
        valueMap.insert("work",QVariant(tmpWorkMap));
    
    // Insert to contact map
    if(valueMap.size() > 0)
        m_contactMap.insert(QString("tel"),QVariant(valueMap));
}

void JSonizer::addressFields2json(const QContact * contact)
{
    QList<QContactDetail> addressList = contact->details(QContactAddress::DefinitionName);
    QList<QContactDetail> emailAddressList = contact->details(QContactEmailAddress::DefinitionName);
    QList<QContactDetail> urlList = contact->details(QContactUrl::DefinitionName);
    
    QMap<QString,QVariant> valueMap;
    QMap<QString,QVariant> tmpHomeMap;
    QMap<QString,QVariant> tmpWorkMap;
    
    for (int i = 0; i < addressList.size(); i++) {
        QContactAddress address = addressList.at(i);
        if (!address.contexts().size()) {
            valueMap.insert("street",QVariant(address.street()));
            valueMap.insert("local",QVariant(address.locality()));
            valueMap.insert("region",QVariant(address.region()));
            valueMap.insert("code",QVariant(address.postcode()));
            valueMap.insert("country",QVariant(address.country()));
        } else {
            if (address.contexts().at(0) == QContactAddress::ContextHome) {
                tmpHomeMap.insert("street",QVariant(address.street()));
                tmpHomeMap.insert("local",QVariant(address.locality()));
                tmpHomeMap.insert("region",QVariant(address.region()));
                tmpHomeMap.insert("code",QVariant(address.postcode()));
                tmpHomeMap.insert("country",QVariant(address.country()));
            } else if (address.contexts().at(0) == QContactAddress::ContextWork) { 
                tmpWorkMap.insert("street",QVariant(address.street()));
                tmpWorkMap.insert("local",QVariant(address.locality()));
                tmpWorkMap.insert("region",QVariant(address.region()));
                tmpWorkMap.insert("code",QVariant(address.postcode()));
                tmpWorkMap.insert("country",QVariant(address.country()));            
            }
        }
    }
    
    for (int j = 0; j < emailAddressList.size(); j++) {
        QContactEmailAddress emailAddress = emailAddressList.at(j);
        if (!emailAddress.contexts().size()) {
            if(valueMap.contains("email")) {
                QStringList mailList;
                if(valueMap.value("email").type() == QVariant::String) {
                    mailList.append(valueMap.value("email").toString());
                    mailList.append(emailAddress.emailAddress());
                } else if(valueMap.value("email").type() == QVariant::List) {
                    mailList = valueMap.value("email").toStringList();
                    mailList.append(emailAddress.emailAddress());
                }
                valueMap.insert("email", QVariant(mailList));
            } else {
                valueMap.insert("email",QVariant(emailAddress.emailAddress()));
            }        
        } else {
            if (emailAddress.contexts().at(0) == QContactEmailAddress::ContextHome) {
                if(tmpHomeMap.contains("email")) {
                    QStringList mailList;
                    if(tmpHomeMap.value("email").type() == QVariant::String) {
                        mailList.append(tmpHomeMap.value("email").toString());
                        mailList.append(emailAddress.emailAddress());
                    } else if(tmpHomeMap.value("email").type() == QVariant::List) {
                        mailList = tmpHomeMap.value("email").toStringList();
                        mailList.append(emailAddress.emailAddress());
                    }
                    tmpHomeMap.insert("email", QVariant(mailList));
                } else {
                    tmpHomeMap.insert("email",QVariant(emailAddress.emailAddress()));
                }            
            } else if (emailAddress.contexts().at(0) == QContactEmailAddress::ContextWork) {
                if(tmpWorkMap.contains("email")) {
                    QStringList mailList;
                    if(tmpWorkMap.value("email").type() == QVariant::String) {
                        mailList.append(tmpWorkMap.value("email").toString());
                        mailList.append(emailAddress.emailAddress());
                    } else if(tmpWorkMap.value("email").type() == QVariant::List) {
                        mailList = tmpWorkMap.value("email").toStringList();
                        mailList.append(emailAddress.emailAddress());
                    }
                    tmpWorkMap.insert("email", QVariant(mailList));
                } else {
                    tmpWorkMap.insert("email",QVariant(emailAddress.emailAddress()));
                }             
            }
        }
    }

    for (int k = 0; k < urlList.size(); k++) {
        QContactUrl url = urlList.at(k);
        if (!url.contexts().size()) {
            valueMap.insert("uri",QVariant(url.url()));
        } else {
            if (url.contexts().at(0) == QContactUrl::ContextHome) {
                tmpHomeMap.insert("uri",QVariant(url.url()));
            } else if (url.contexts().at(0) == QContactUrl::ContextWork) {
                tmpWorkMap.insert("uri",QVariant(url.url()));
            }
        }
    }
    
    /*addressList.clear();
    emailAddressList.clear();
    urlList.clear();*/
    if(tmpHomeMap.size() > 0)
        valueMap.insert("home", QVariant( tmpHomeMap));
    if(tmpWorkMap.size() > 0)
        valueMap.insert("work", QVariant( tmpWorkMap));
    
    /*tmpHomeMap.clear();
    tmpWorkMap.clear();*/
    
    // Insert addresses
    if(valueMap.size() > 0)
        m_contactMap.insert(QString("address"),QVariant(valueMap));
    
    //valueMap.clear();
}

void JSonizer::companyFields2json(const QContact * contact)
{
    QContactOrganization organization = contact->detail(QContactOrganization::DefinitionName);
    QMap<QString,QVariant> valueMap;
    if(!((organization.name()).isEmpty()))
        valueMap.insert("name",QVariant(organization.name()));
    if(!((organization.title()).isEmpty()))
        valueMap.insert("title",QVariant(organization.title()));

    // Insert company information
    if(valueMap.size() > 0)
        m_contactMap.insert(QString("company"),QVariant(valueMap));
}

void JSonizer::miscFields2json(const QContact * contact)
{
    m_contactMap.insert(QString("id"),QVariant(contact->id().localId()));

    QContactNote note = contact->detail(QContactNote::DefinitionName);
    if(!((note.note()).isEmpty()))
        m_contactMap.insert(QString("notes"),QVariant(note.note()));

    QContactAnniversary anniversary = contact->detail(QContactAnniversary::DefinitionName);
    if(!((anniversary.originalDate()).isNull()))
        m_contactMap.insert(QString("anniversary"),QVariant(anniversary.originalDate()));   

    QContactBirthday birthday = contact->detail(QContactBirthday::DefinitionName);
    if(!((birthday.date()).isNull()))
        m_contactMap.insert(QString("birthday"),QVariant(birthday.date().toString("ddd MMMM d yyyy")));

    QContactNickname nickname = contact->detail(QContactNickname::DefinitionName);
    if(!((nickname.nickname()).isEmpty()))
        m_contactMap.insert(QString("nickname"),QVariant(nickname.nickname()));

    QContactAvatar avatar = contact->detail(QContactAvatar::DefinitionName);
    if(!((avatar.imageUrl()).isEmpty()))
        m_contactMap.insert(QString("photo"),QVariant(avatar.imageUrl()));

    QContactDisplayLabel displayName = contact->detail(QContactDisplayLabel::DefinitionName);
    if(!((displayName.label()).isEmpty()))
        m_contactMap.insert(QString("displayName"),QVariant(displayName.label()));

    
    QContactGeoLocation position = contact->detail(QContactGeoLocation::DefinitionName);

    if(!((position.label()).isEmpty()))
        m_contactMap.insert(QString("position"),QVariant(position.label()));
    if(position.latitude())
        m_contactMap.insert(QString("positionCoordinatesLatitude"),QVariant(QString::number(position.latitude())));
    if(position.longitude())
        m_contactMap.insert(QString("positionCoordinatesLongitude"),QVariant(QString::number(position.longitude())));        
    if(position.altitude())
        m_contactMap.insert(QString("positionCoordinatesAltitude"),QVariant(QString::number(position.altitude())));        
    if(position.accuracy())
        m_contactMap.insert(QString("positionCoordinatesAccuracy"),QVariant(QString::number(position.accuracy())));        
    if(position.altitudeAccuracy())
        m_contactMap.insert(QString("positionCoordinatesAltitudeAccuracy"),QVariant(QString::number(position.altitudeAccuracy())));        
    if(position.heading())
        m_contactMap.insert(QString("positionCoordinatesHeading"),QVariant(QString::number(position.heading())));        
    if(position.speed())
        m_contactMap.insert(QString("positionCoordinatesSpeed"),QVariant(QString::number(position.speed())));        
    if(!(((position.timestamp()).toString()).isEmpty()))
        m_contactMap.insert(QString("positionTimestamp"),QVariant(position.timestamp().toString()));        

        
  /*  QList<QVariant> xspidList;
    xspidList.append(position.label());
    xspidList.append(QString::number(position.latitude()));
    xspidList.append(QString::number(position.longitude()));
    xspidList.append(QString::number(position.altitude()));
    xspidList.append(QString::number(position.accuracy()));
    xspidList.append(QString::number(position.altitudeAccuracy()));
    xspidList.append(QString::number(position.heading()));
    xspidList.append(QString::number(position.speed()));
    xspidList.append(position.timestamp().toString());
    // Append strings to xspid key
    m_contactMap.insert(QString("xspid"),QVariant(xspidList));*/

}

