/*
 * Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
 *
 * This file is part of Qt Web Runtime.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include <QtCore>

#include <QServiceInterfaceDescriptor>
#include <QServiceContext>
#include <QAbstractSecuritySession>

#include "qmessagingserviceplugin.h"
#include "qmessagingservice.h"


QMessagingServicePlugin::QMessagingServicePlugin()
{
}

/*!
    Destructor
 */
QMessagingServicePlugin::~QMessagingServicePlugin()
{
}

/*!
    This API is used to instantiate Reference Service service provider

    \param refCounted true reference counted object.
    \param secSession Security Manager session - not owned
    @return interface pointer to Reference Service object
 */

QObject* QMessagingServicePlugin::createInstance(const QServiceInterfaceDescriptor& descriptor,
                                                 QServiceContext* context,
                                                 QAbstractSecuritySession* session) 
{
	Q_UNUSED(context);

        QMessagingService *msgSrv = new QMessagingService(this);
        msgSrv->setSecuritySession(session);

        if (descriptor.interfaceName() == KIMessagingId)
            return msgSrv;
        else
            return 0;

}

Q_EXPORT_PLUGIN(QMessagingServicePlugin);
