#include <QLocale>
#include <QString>
#include <QRect>
#include <QApplication>
#include <QDesktopWidget>
#include <QClipboard>

#include "qdeviceinfo.h"
#include "qsysinfoproperties.h"
#include "qsysinfoelement.h"

using namespace DeviceInfo;
using namespace DeviceInfo::ChannelPhoneLanguage;
using namespace DeviceInfo::ChannelDisplayOrientation;
using namespace DeviceInfo::ChannelClipBoard;
using namespace DeviceInfo::ChannelDisplayResolution;

QDeviceInfo::QDeviceInfo()
{
}

int QDeviceInfo::getChannel(SysInfoProperty channelId ,
        const ChannelInformation& input,
        QMap<SysInfoProperty, QVariant>& channelInfo)
{
    QDesktopWidget* desktop = QApplication::desktop();
    int errCode = ErrNoError;

    switch( channelId )
    {
        case PhoneLanguage:
            {
                QLocale locale = QLocale::system();
                QString lName = locale.name().replace( QChar('_'), "-" );
                channelInfo.insert( phoneLanguage , lName);
                break;
            }
        case DisplayOrientation:
            {
                int h, w;

                if( desktop )
                {
                    QRect currentGeometry = desktop->screenGeometry();
                    h = currentGeometry.height();
                    w = currentGeometry.width();

                    if(h > w)
                        channelInfo.insert( displayOrientation ,  "Landscape" );
                    else
                        channelInfo.insert( displayOrientation , "Portrait" );
                }
                else
                    errCode = -1;
                break;
            }
        case ClipBoard:
            {
                QClipboard *clipbrd = QApplication::clipboard();
                if(clipbrd)
                {
                    QString text = clipbrd->text(QClipboard::Clipboard);
                    channelInfo.insert(clipBoardData , text  );
                }
                else
                    errCode = -1;
                break;
            }
        case DisplayResolution:
            {
                int h = desktop->height();
                int w = desktop->width();

                if ( width > height )
                {
                    channelInfo.insert( height , h );
                    channelInfo.insert( width , w );
                }
                else
                {
                    channelInfo.insert(height,w);
                    channelInfo.insert(width,h);
                }
                break;
            }
        default:
            {
                errCode = ErrNotSupported;
                channelInfo.insert( channelId, "" );
                break;
            }
    }
    return errCode;
}

int QDeviceInfo::getChannelAsync( SysInfoProperty  , int )
{
    return 0;
}

int QDeviceInfo::startChannel( SysInfoProperty ,
        const ChannelInformation& ,
        int )
{

    return 0;
}
int QDeviceInfo::stopChannel( SysInfoProperty ) 
{
    return 0;

}
int QDeviceInfo::setChannelSync( SysInfoProperty channelId ,  const QVariant& input, QMap<SysInfoProperty, QVariant>& errData)
{
    int errCode = ErrNoError;

    switch( channelId )
    {
        case ClipBoard:
            {
                QString text = input.toString();
                if( input.isNull()  == false)
                {
                    if( text.isEmpty() )
                    {
                        errCode = ErrMissingArgument;
                        errData.insert( clipBoardData, "" );
                        break;
                    }
                    QClipboard *clipboard = QApplication::clipboard();
                    clipboard->setText( text , QClipboard::Clipboard );
                }
                else
                {
                    errCode = ErrMissingArgument;
                    errData.insert( clipBoardData, "" );
                }
                break;
            }
        default:
            errCode = ErrNotSupported;
            errData.insert( channelId, "" );
            break;
    }
    return errCode;
}

int QDeviceInfo::cancel( SysInfoProperty  , int )
{

    return 0;
}

int QDeviceInfo::getChannelList(QList<SysInfoProperty>& channelList) 
{
    channelList += ClipBoard ;
    channelList += PhoneLanguage;
    channelList += DisplayResolution;
    channelList += DisplayOrientation;

    return 0;
}
