#include <QString>
#include <QX11Info>
#include <QFile>
#include <QDBusMessage>
#include <QDBusInterface>
#include <QDesktopWidget>
#include <QApplication>

#include "qdeviceinfo_p.h"
#include "qsysinfoproperties.h"
#include "qsysinfoelement.h"
//#include "qpublicidreciever.h"

#define SYSINFO_PRODUCT_TYPE            "/component/product"
//SystemInfo
#define SYSINFO_SERVICE                 "com.nokia.SystemInfo"
#define SYSINFO_PATH                    "/com/nokia/SystemInfo"
#define SYSINFO_IF                      "com.nokia.SystemInfo"
#define SYSINFO_GET_CONFIG              "GetConfigValue"
#define SYSINFO_FIRMWARE                "/device/sw-release-ver"
#define SYSINFO_PHONE_MODEL             "/component/product-name"


//CountryName
const QString CLOCKD_SERVICE("com.nokia.clockd");
const QString CLOCKD_PATH("/com/nokia/clockd");
const QString CLOCKD_INTERFACE("com.nokia.clockd");
const QString GET_TZ("get_tz");

#define SIM_SERVICE_IMEI                "com.nokia.csd.Info"
#define SIM_PATH_IMEI                   "/com/nokia/csd/info"
#define SIM_IF_IMEI                     "com.nokia.csd.Info"
#define SIM_GET_IMEI                    "GetIMEINumber"

//Hardware
#define HAL_HARD_SERVICE                "org.freedesktop.Hal"
#define HAL_HARD_PATH                   "/org/freedesktop/Hal/devices/computer"
#define HAL_HARD_IF                     "org.freedesktop.Hal.Device"
#define HAL_HARD_GET_PROPERTY           "GetProperty"
#define HAL_HARD_MANUFACTURER           "system.hardware.vendor"

//PublicId
const QString INFO_SERVICE("com.nokia.csd.Info");
const QString INFO_PATH("/com/nokia/csd/info");
const QString INFO_INTERFACE("com.nokia.csd.Info");
const QString INFO_GETPUBLICID("GetPublicId");

using namespace DeviceInfo;
using namespace DeviceInfo::ChannelIMEI;
using namespace DeviceInfo::ChannelPhoneLanguage;
using namespace DeviceInfo::ChannelProductType;
using namespace DeviceInfo::ChannelFirmwareVersion;
using namespace DeviceInfo::ChannelPhoneColorDepth;
using namespace DeviceInfo::ChannelDisplayResolution;
using namespace DeviceInfo::ChannelPlatformName;
using namespace DeviceInfo::ChannelPhoneModel;
using namespace DeviceInfo::ChannelPublicId;
using namespace DeviceInfo::ChannelPhoneManufacturer;
using namespace DeviceInfo::ChannelCountryName;


QString QDeviceInfoPrivate::getSysinfoKey(char* key)
{
    QString KeyValue = "";
    /*
    struct system_config *sc = 0;
    
    if( sysinfo_init(&sc) == 0 )
    {
        uint8_t *data = 0;
        unsigned long size = 0;
        if( sysinfo_get_value(sc, key, &data, &size) == 0 )
        {
            unsigned long k;
            for( k = 0; k < size; ++k )
            {
                int c = data[k];
                char cc;
                if( c < 32 || c > 126 )
                    sprintf(&cc, "#%02x", c);
                else
                    sprintf(&cc,"%c", c);
                KeyValue.append(cc);

            }
            free(data);
        }
    }
    sysinfo_finish(sc);*/
    return KeyValue;
}

QDeviceInfoPrivate::QDeviceInfoPrivate()
{
}

int QDeviceInfoPrivate::getChannel(SysInfoProperty channelId ,
        const ChannelInformation& input,
        QMap<SysInfoProperty, QVariant>& channelInfo)
{
    
    int errCode = ErrNoError;

    switch( channelId )
    {
        case ProductType:
            {
                QDBusInterface interface( SYSINFO_SERVICE,
                        SYSINFO_PATH,
                        SYSINFO_IF,
                        QDBusConnection::systemBus());
                if ( FALSE == interface.isValid() )
                {
                    errCode = ErrServiceNotAvailable; 
                    channelInfo.insert( channelId," " );
                    break;
                }
                QDBusMessage msg = interface.call( SYSINFO_GET_CONFIG,
                        SYSINFO_PRODUCT_TYPE );
                if ( msg.type() == QDBusMessage::ReplyMessage )
                {
                    QString product_type = msg.arguments().at(0).toByteArray();
                    channelInfo.insert( productType , product_type);
                }
                break;
            }
        case FirmwareVersion:
            {
                QDBusInterface interface( SYSINFO_SERVICE,
                        SYSINFO_PATH,
                        SYSINFO_IF,
                        QDBusConnection::systemBus());
                if ( FALSE == interface.isValid() )
                {
                    errCode = ErrServiceNotAvailable; 
                    channelInfo.insert( channelId," " );
                    break;
                }
                QDBusMessage msg = interface.call( SYSINFO_GET_CONFIG,
                        SYSINFO_FIRMWARE );
                if ( msg.type() == QDBusMessage::ReplyMessage )
                {
                    QString firmware_version = msg.arguments().at(0).toByteArray();
                    channelInfo.insert( firmwareVersion , firmware_version );
                }
                else
                {
                    errCode = ErrServiceNotAvailable; 
                    channelInfo.insert( channelId," " );
                    break;
                }
                break;
            }
        case CountryName:
            {
                QDBusInterface interface( CLOCKD_SERVICE,
                        CLOCKD_PATH,
                        CLOCKD_INTERFACE,
                        QDBusConnection::systemBus());
                if ( FALSE == interface.isValid() )
                {
                    errCode = ErrServiceNotAvailable;
                    break;
                }

                QDBusMessage msg = interface.call( GET_TZ );
                if ( msg.type() == QDBusMessage::ReplyMessage &&
                        msg.arguments().count() >= 1 &&
                        msg.arguments().at(0).type() == QVariant::String )
                {
                    QString timezone = msg.arguments().at(0).toString().remove(QChar(':'));
                    QFile zone("/usr/share/zoneinfo/zone.tab");
                    QMap<QString,QString> zoneMap;
                    if ( TRUE == zone.open(QFile::ReadOnly) )
                    {
                        while ( FALSE == zone.atEnd() )
                        {
                            QByteArray line = zone.readLine();
                            if (line.at(0)=='#')
                            {
                                continue;
                            }

                            QList<QByteArray> vars = line.split('\t');
                            if ( vars.count() >= 3 )
                            {
                                zoneMap.insert(vars.at(2).trimmed(),vars.at(0).trimmed());//time zone ,country
                            }
                        }
                        zone.close();

                        channelInfo.insert(countryName,zoneMap.value(timezone));
                    }
                    else
                    {
                        channelInfo.insert(countryName,QString("GB"));
                    }
                }
                else
                {
                    errCode = ErrServiceNotAvailable;
                    break;
                }
                break;
            }
        case IMEI:
            {
                QDBusInterface interface( SIM_SERVICE_IMEI,
                        SIM_PATH_IMEI,
                        SIM_IF_IMEI,
                        QDBusConnection::systemBus());
                if ( FALSE == interface.isValid() )
                {
                    errCode = ErrDataNotFound; 
                    channelInfo.insert( channelId," " );
                    break;
                }
                QDBusMessage msg = interface.call( SIM_GET_IMEI );
                if ( msg.type() == QDBusMessage::ReplyMessage )
                {
                    QString imei_res = msg.arguments().at(0).toString();
                    channelInfo.insert( imei , imei_res );
                }
                else
                {
                    errCode = ErrServiceNotAvailable; 
                    channelInfo.insert( channelId," " );
                    break;
                }
                break;
            }
        case PhoneColorDepth:
            {
                QDesktopWidget* desktop = QApplication::desktop();
                QDesktopWidget widget;          
                qint32 depth = widget.screen(desktop->primaryScreen())->x11Info().depth();
                channelInfo.insert( phoneColorDepth , depth );
                break;
            }
        case PhoneManufacturer:
            {
                QDBusInterface interface( HAL_HARD_SERVICE,
                        HAL_HARD_PATH,
                        HAL_HARD_IF,
                        QDBusConnection::systemBus());
                if ( FALSE == interface.isValid() )
                {
                    errCode = ErrServiceNotAvailable; 
                    channelInfo.insert( channelId," " );
                    break;
                }
                QDBusMessage msg = interface.call( HAL_HARD_GET_PROPERTY,
                        HAL_HARD_MANUFACTURER );
                if ( msg.type() == QDBusMessage::ReplyMessage )
                {
                    QString manufacturer = msg.arguments().at(0).toString();
                    channelInfo.insert( phoneManufacturer , manufacturer  );
                    break;
                }
                else
                {
                    errCode = ErrServiceNotAvailable; 
                    channelInfo.insert( channelId," " );
                    break;
                }
                break;
            }
        case PhoneModel:
            {
                QString model = "N900";
                channelInfo.insert(phoneModel,model );
                break;
            }
        case PlatformName:
            {
                QString platform_name = "Maemo5";
                channelInfo.insert( platformName, platform_name );
                break;
            }
        case PublicId:
            {
                QDBusInterface interface( INFO_SERVICE,
                        INFO_PATH,
                        INFO_INTERFACE,
                        QDBusConnection::systemBus());
                if ( true == interface.isValid() )
                {
                    QDBusMessage msg = interface.call( INFO_GETPUBLICID );
                    if ( msg.type() == QDBusMessage::ReplyMessage &&
                            msg.arguments().count() >= 1 &&
                            msg.arguments().at(0).type() == QVariant::ByteArray )
                    {
                        QString pubId;
                        QByteArray var = msg.arguments().at(0).toByteArray();
                        for ( int i=0; i < var.size(); i++ )
                        {
                            pubId.append(QString("%1").arg(var[i], 2, 16, QChar('0')));
                        }
                        channelInfo.insert( publicId,pubId );
                    }
                    else
                    {
                        errCode = ErrServiceNotAvailable;
                    }
                }
                else
                {
                    errCode = ErrServiceNotAvailable;
                }
                break;
            }
    default:
    {
        errCode = QDeviceInfo::getChannel( channelId,input,channelInfo );
        break;
    }
}
return errCode;
}

int QDeviceInfoPrivate::getChannelAsync( SysInfoProperty channelId , int transId)
{
    int errCode = ErrNoError;

    switch( channelId )
    {
        default:
            errCode = QDeviceInfo::getChannelAsync( channelId, transId ) ;
    }
    return errCode;
}

int QDeviceInfoPrivate::startChannel( SysInfoProperty channelId,
        const ChannelInformation& triggerInfo,
        int transId)
{

    int errCode = ErrNoError;

    switch( channelId )
    {
        default:
            errCode = QDeviceInfo::startChannel( channelId, triggerInfo, transId ) ;
    }
    return errCode;
}

int QDeviceInfoPrivate::stopChannel( SysInfoProperty channelId ) 
{
    int errCode = ErrNoError;

    switch( channelId )
    {
        default:
            errCode = QDeviceInfo::stopChannel( channelId) ;
    }
    return errCode;
}

int QDeviceInfoPrivate::setChannelSync( SysInfoProperty channelId ,  const QVariant& input, QMap<SysInfoProperty, QVariant>& errData)
{
    int errCode = ErrNoError;

    switch( channelId )
    {
        default:
            errCode = QDeviceInfo::setChannelSync( channelId, input, errData) ;
    }
    return errCode;
}

int QDeviceInfoPrivate::cancel( SysInfoProperty channelId , int transId)
{
    int errCode = ErrNoError;

    switch( channelId )
    {
        default:
            errCode = QDeviceInfo::cancel( channelId, transId) ;
    }
    return errCode;
}

int QDeviceInfoPrivate::getChannelList(QList<SysInfoProperty>& channelList) 
{
    channelList += IMEI ;
    channelList += ProductType;
    channelList += FirmwareVersion ;
    channelList += PhoneColorDepth ;
    channelList += PhoneManufacturer; 
    channelList += PhoneModel ;
    channelList += PlatformName;  
    channelList += PublicId;
    channelList += CountryName;

    QDeviceInfo::getChannelList( channelList );
    return 0;
}
