#include "qkeystore.h"
#include "qsysinfoelement.h"
#include "qsysinfoerrors.h"

#include <QStringList>

QHash<QString, SysInfoProperty>* QKeyStore::hashTable = NULL;
QHash<SysInfoProperty,ChannelProperty >* QKeyStore::propertyTable = NULL;
int QKeyStore::refCount = 0;

QKeyStore::QKeyStore( const QString& key, 
                      SysInfoProperty value, 
                      int operationType,
                      int responseType)
{
    if( !hashTable )
    {    
        qRegisterMetaType<ChannelInformation>("ChannelInformation");        
        hashTable = new QHash<QString, SysInfoProperty>();
    }

    if( hashTable->value( key ) != value ) 
        hashTable->insertMulti( key, value );
    
    if( !propertyTable )
        propertyTable = new QHash<SysInfoProperty,ChannelProperty >();
    
    if( !propertyTable->contains( value ) )
    {
        ChannelProperty prop ;
        prop.propType = TypeChannel;
        prop.channelData.operationType = operationType;
        prop.channelData.responseType = responseType;

        propertyTable->insertMulti(value,prop);
    }
    refCount++;
}

QKeyStore::QKeyStore( const QString& key, 
               SysInfoProperty value ,
               int parent ,
               QVariant::Type type ,
               const QString& range,
               const QString& description)
{
    if( !hashTable )
    {    
        qRegisterMetaType<ChannelInformation>("ChannelInformation");        
        hashTable = new QHash<QString, SysInfoProperty>();
    }
    if( hashTable->value( key ) != value )   
        hashTable->insertMulti( key, value );

    if( !propertyTable )
        propertyTable = new QHash<SysInfoProperty,ChannelProperty >();

    if( !propertyTable->contains( value ) )
    {
        ChannelProperty prop ;
        prop.propType = TypeProperty;
        prop.propertyData.type = type;
        prop.propertyData.range = range;
        prop.propertyData.description = description;

        ChannelProperty& parentProp = (*propertyTable)[(SysInfoProperty)parent];

        if( parentProp.propType != TypeInvalid )
        {
            parentProp.properties.append( value );
        }

        propertyTable->insertMulti(value,prop);
    }
    refCount++;
    
}

QKeyStore::~QKeyStore()
{
    if( --refCount == 0 )
    {
        delete hashTable;
        delete propertyTable;
    }
}

SysInfoProperty QKeyStore::value(const QString& key)
{
	return hashTable->value(key);

}

const QString QKeyStore::key(SysInfoProperty value)
{
	return hashTable->key(value);
}

int QKeyStore::propertySchema(SysInfoProperty p, QMap<QString,QVariant>& schema)
{
    int err = ErrNoError;
    
    ChannelProperty prop = (*propertyTable)[ p ];

    switch( prop.propType )
    {
        case TypeInvalid:
            err = ErrDataNotFound;
            break;
        case TypeChannel:
            {
                QStringList style;
                if( prop.channelData.operationType & TypeSynchrounous )
                    style << "Sync";
                else
                    style << "";

                if( prop.channelData.operationType & TypeAsynchrounous )
                    style << "Async";
                else
                    style << "";
                
                if( prop.channelData.responseType & TypeOneShot )
                    style << "OneShot";
                else
                    style << "";
                
                if( prop.channelData.responseType & TypeNotification )
                    style << "Notification";
                else
                    style << "";

                schema.insert("style", style );
                break;
            }
        case TypeProperty:
            {
                schema.insert("range", prop.propertyData.range );
                schema.insert("description", prop.propertyData.description );
                break;
            }

        default:
            break;
    }

    if( err == ErrNoError )
    {
        QList< QVariant > data;
        for( int i=0; i < prop.properties.size(); i++ )
        {
            QMap<QString, QVariant> pschema;
            propertySchema( prop.properties.at(i), pschema );
            data.append( pschema );
        }


        schema.insert("name",QKeyStore::key(p));
        if( data.size() )
            schema.insert("data", data );
    }
    return err;
}
