#include <QX11Info>
#include <gconf/gconf.h>
#include <gconf/gconf-client.h>
#include <gst/gst.h>
#include "qmultimedia_p.h"
#include "qsysinfoerrors.h"
#include "qsysinfoproperties.h"
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>

using namespace Multimedia;
using namespace Multimedia::ChannelSupportedVideoDecoders;
using namespace Multimedia::ChannelDisplayWallpaper;

//Codec defines
const QLatin1String  GST_CODEC_TYPE_VIDEO("Codec/Decoder/Video");

int QMultimediaPrivate::getChannel( SysInfoProperty channelId , 
            const ChannelInformation& input ,
            QMap<SysInfoProperty, QVariant>& channelInfo )
{
    int errCode = ErrNoError;
    QSysInfoElement element;
    
    switch( channelId )
    {
        case SupportedVideoDecoders:
            {
                //VideoCodecList availableMimeTypes;
                QList< QVariant > availableMimeTypesForSP;

                if ( TRUE == gst_init_check(NULL,NULL,NULL) )
                {
                    // Iterate over all audio and video decoders and extract mime types from sink caps
                    GList* factoryList = gst_registry_get_feature_list(gst_registry_get_default (), GST_TYPE_ELEMENT_FACTORY);

                    for ( GList* iter = (factoryList) ; iter != NULL ; iter = iter->next )
                    {
                        GstPluginFeature *feature = GST_PLUGIN_FEATURE(iter->data);
                        if ( NULL != feature)
                        {
                            QString klass = gst_element_factory_get_klass(GST_ELEMENT_FACTORY(feature));

                            if ( klass == QLatin1String(GST_CODEC_TYPE_VIDEO) )
                            {
                                const GList *static_templates = NULL;
                                GstElementFactory *factory = GST_ELEMENT_FACTORY(feature);
                                if ( NULL != factory )
                                {
                                    static_templates = gst_element_factory_get_static_pad_templates(factory);

                                    for (; static_templates != NULL ; static_templates = static_templates->next )
                                    {
                                        GstStaticPadTemplate *padTemplate = (GstStaticPadTemplate *) static_templates->data;
                                        if ( padTemplate &&
                                                padTemplate->direction == GST_PAD_SINK )
                                        {
                                            GstCaps *caps = gst_static_pad_template_get_caps (padTemplate);

                                            if ( caps )
                                            {
                                                QMap <QString, QVariant> mimeDetails;
                                                mimeDetails.insert( "manufacturer",gst_element_factory_get_author (GST_ELEMENT_FACTORY(feature)) );
                                                mimeDetails.insert( "identifier",QString::fromUtf8(gst_structure_get_name (gst_caps_get_structure (caps, 0))) );
                                                mimeDetails.insert( "maxBitrate","" );
                                                mimeDetails.insert( "isHWAccelerationEnabled","" );

                                                if ((GST_PLUGIN_FEATURE (iter->data))->plugin_name)
                                                {
                                                    GstPlugin *plugin = gst_registry_find_plugin (gst_registry_get_default (),
                                                            GST_PLUGIN_FEATURE (iter->data)->plugin_name);
                                                    mimeDetails.insert( "version", (plugin)?(plugin->desc.version):("N/A") );
                                                }
                                                availableMimeTypesForSP.append(static_cast<QVariant>(mimeDetails));
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                    if ( NULL != factoryList )
                        g_list_free( factoryList );
                }

                channelInfo.insert(supportedVideoDecoders,static_cast<QVariant>(availableMimeTypesForSP));
                break;
            }
        default:
            break;
    }
    return errCode;
}

int QMultimediaPrivate::getChannelAsync( SysInfoProperty channelId , int transId)
{
    return 0;
}

int QMultimediaPrivate::startChannel( SysInfoProperty channelId,
                                 const ChannelInformation& triggerInfo,
                                 int transId)
{
    return 0;
}
int QMultimediaPrivate::stopChannel( SysInfoProperty channelId) 
{
return 0;

}
int QMultimediaPrivate::setChannelSync( SysInfoProperty channelId ,  const QVariant& input, QMap<SysInfoProperty, QVariant>& errData )
{
    
    int errCode = ErrNoError;

    switch(channelId)
    {
        case DisplayWallpaper:
            {
                const QString display_name = getenv( "DISPLAY" );
                Display *display = XOpenDisplay( display_name.toAscii().data() );
                Window   rootWindow = RootWindow( display, DefaultScreen(display) );
                Atom     atom = XInternAtom( display, "_NET_CURRENT_DESKTOP", FALSE );

                Atom realType;
                int format,status;
                unsigned long n, extra;
                unsigned char* data = NULL;
                status = XGetWindowProperty(display,
                        rootWindow,
                        atom,
                        0L, 16L, 0,
                        XA_CARDINAL,
                        &realType,
                        &format,
                        &n,
                        &extra,
                        &data );
                qint32 num = -1;
                if ( status == Success &&
                        realType == XA_CARDINAL &&
                        format == 32 &&
                        n == 1 &&
                        data )
                {
                    num = ((qint32*)data)[0];
                }


                if ( NULL != data )
                {
                    XFree(data);
                }
                XCloseDisplay( display );

                const QString uri = input.toString();
                if(uri.isEmpty())
                {
                    errCode = ErrUriNotFound;
                    errData.insert( wallpaperFileurl, "NULL uri" );
                    break;
                }

                GConfClient *client = gconf_client_get_default();
                if ( NULL != client )
                {
                    const QString path = g_strdup_printf("/apps/osso/hildon-desktop/views/%i/bg-image",1 + num);
                    if ( !gconf_client_set_string(client, path.toAscii().data(), uri.toAscii().data(), NULL) )
                    {
                        errCode = ErrInvalidArgument;
                        errData.insert( wallpaperFileurl, "gconf_client_set_string error" );
                    }
                    g_object_unref(client);
                }
                break;
            }
        default:
            errCode = ErrNotSupported;
            errData.insert( channelId, "" );
            break;
    }
    return errCode;
}
int QMultimediaPrivate::cancel( SysInfoProperty channelId , int transId)
{

    return 0;
}
int QMultimediaPrivate::getChannelList(QList<SysInfoProperty>& channelList) 
{
    channelList += DisplayWallpaper;
    channelList += SupportedVideoDecoders;
    return 0;
}
