#include <QDBusMessage>
#include <QDBusInterface>
#include <QDBusArgument>
#include <QDBusConnection>
#include <QDBusReply>
#include <QThreadPool>
#include <QProcess>
#include <qnetworksession.h>
#include <qnetworkconfigmanager.h>

#include "qnetworkinfo_p.h"
#include "qsysinfoproperties.h"
#include "qsysinfoelement.h"
#include "qkeystore.h"
#include "qasynctask.h"


//Home Network
const QString HN_SERVICE("com.nokia.phone.SIM");
const QString HN_PATH("/com/nokia/phone/SIM");
const QString HN_IF("Phone.Sim");
//const QString HN_GET_OPERATOR_NAME("get_service_provider_name");
//const QString HN_GET_SERVICE_INFO("read_hplmn");

#define CSNET_SERVICE                           "com.nokia.csd"
#define CSNET_PATH                              "/com/nokia/csd/csnet"
#define CSNET_NET_REGISTRATION_IF               "com.nokia.csd.CSNet.NetworkRegistration"
#define CSNET_SIGNAL_STRENGTH_PROPERTY  "SignalBars"
//SIM
#define SIM_SERVICE_IMSI                "com.nokia.csd.SIM"
#define SIM_PATH_IMSI                   "/com/nokia/csd/sim"
#define SIM_IF_IMSI                     "com.nokia.csd.SIM.Identity"
#define SIM_GET_IMSI                    "GetIMSI"

#define PROPERTY_IF                  "org.freedesktop.DBus.Properties"
#define PROPERTY_GET                  "Get"
//CSNET: used by SignalStrength, Network
#define CSNET_IF                     "com.nokia.csd.CSNet.SignalStrength"
#define CSNET_SIGNAL_STRENGTH_PROPERTY  "SignalBars"
#define CSNET_CHANGE_SIGNAL_STRENGTH_SIGNAL  "SignalStrengthChanged"

//HAL
#define HAL_IM_SERVICE                  "org.freedesktop.Hal"
#define HAL_IM_PATH                     "/org/freedesktop/Hal/Manager"
#define HAL_IM_IF                       "org.freedesktop.Hal.Manager"
#define HAL_IM_FIND_DEVICE              "FindDeviceByCapability"
//Wireless IFs
#define HAL_WI_WLAN                   "net.80211"
#define HAL_WI_IRDA                   "net.irda"
#define HAL_WI_BT                     "net.bluetooth"
#define CSNET_NET_RA_IF                         "com.nokia.csd.CSNet.RadioAccess"
#define CSNET_NET_RA_TECHNOLOGY                 "Technology"

#define PHONENET_SERVICE            "com.nokia.phone.net"
#define PHONENET_PATH               "/com/nokia/phone/net"
#define PHONENET_IF                 "Phone.Net"
#define PHONENET_GET_REGISTR_STATUS "get_registration_status"

#define PHONENET_GET_MODE           "get_radio_access_technology"
#define PHONENET_GET_OPERATOR_NAME  "get_operator_name"

#define PHONENET_GET_SIGNAL_STRENGTH "get_signal_strength"
#define PHONENET_CHANGE_SIGNAL_STRENGTH "signal_strength_change"
#define PHONENET_REGISTR_STATUS_CHANGED "registration_status_change"

#define WCS_SERVICE "com.nokia.wirelessconnsupp"
#define WCS_PATH    "/com/nokia/wirelessconnsupp"
#define WCS_IF      "com.nokia.wirelessconnsupp"
#define WCS_GET     "getWirelessConnSupp"
//WirelessConnSupport
#define HAL_WCS_SERVICE "org.freedesktop.Hal"
#define HAL_WCS_PATH    "/org/freedesktop/Hal/Manager"
#define HAL_WCS_IF      "org.freedesktop.Hal.Manager"
#define HAL_WCS_FIND_DEVICE "FindDeviceByCapability"
//Wireless Connection output literals
#define WI_NAME_BLUETOOTH  "Bluetooth"
#define WI_NAME_IRDA  "Irda" 
#define WI_NAME_WLAN  "Wlan" 
#define WI_NAME_GSM "Gsm" 
#define WI_NAME_WCDMA "Wcdma" 
#define WI_NAME_CDMA "Cdma" 

//BlueZ
#define BLUEZ_SERVICE  "org.bluez"
#define BLUEZ_PATH "/"
#define BLUEZ_IF_MANAGER "org.bluez.Manager"
#define BLUEZ_GET_PATH "DefaultAdapter"

typedef enum
{
    NETWORK_REG_STATUS_HOME = 0,                        // CS is registered to home network
    NETWORK_REG_STATUS_ROAM,                            // CS is registered to some other network than home network
    NETWORK_REG_STATUS_ROAM_BLINK,                      // CS is registered to non-home system in a non-home area ('a' or 'b' area)
    NETWORK_REG_STATUS_NOSERV,                          // CS is not in service
    NETWORK_REG_STATUS_NOSERV_SEARCHING,                // CS is not in service, but is currently searching for service
    NETWORK_REG_STATUS_NOSERV_NOTSEARCHING,             // CS is not in service and it is not currently searching for service
    NETWORK_REG_STATUS_NOSERV_NOSIM,                    // CS is not in service due to missing SIM or missing subscription
    NETWORK_REG_STATUS_POWER_OFF = 8,                   // CS is in power off state
    NETWORK_REG_STATUS_NSPS,                            // CS is in No Service Power Save State (currently not listening to any cell)
    NETWORK_REG_STATUS_NSPS_NO_COVERAGE,                // CS is in No Service Power Save State (CS is entered to this state because there is no network coverage)
    NETWORK_REG_STATUS_NOSERV_SIM_REJECTED_BY_NW        // CS is not in service due to missing subscription
} network_reg;

typedef enum
{
    NETWORK_TYPE_HOME = 0,                              //Home network
    NETWORK_TYPE_PREFERRED,                             //Preferred network
    NETWORK_TYPE_FORBIDDEN,                             //Forbidden network
    NETWORK_TYPE_NOPREF_NOFORBIDDEN,                    //Neither preferred nor forbidden network
    NETWORK_TYPE_NO_AVAILABLE                           //No network available
} network_type;

typedef enum
{
    NETWORK_MODE_DUAL = 0,
    NETWORK_MODE_GSM,
    NETWORK_MODE_3G
} network_mode;

typedef enum
{
    NETWORK_HARDCODED_LATIN_OPER_NAME = 0,
    NETWORK_HARDCODED_USC2_OPER_NAME,
    NETWORK_NITZ_SHORT_OPER_NAME,
    NETWORK_NITZ_FULL_OPER_NAME
} network_name;

using namespace NetworkInfo;
using namespace NetworkInfo::ChannelNetwork;
using namespace NetworkInfo::ChannelHomeNetwork;
using namespace NetworkInfo::ChannelSignalStrength;
using namespace NetworkInfo::ChannelIMSI;
using namespace NetworkInfo::ChannelCellularNetworkStatus;
using namespace NetworkInfo::ChannelWirelessConnSupport;
using namespace NetworkInfo::ChannelActiveDataConnections;

QNetworkInfoPrivate::~QNetworkInfoPrivate()
{
    deleteWorkers();
}

QNetworkInfoPrivate::QNetworkInfoPrivate()
{
}

void QNetworkInfoPrivate::handleRequestCompleted (int error, int transactionId,
        const ChannelInformation& channelInfo,
        QNwDBusWorker* worker) 
{   
    emit requestCompleted(error,transactionId,channelInfo) ;
    criticalRegion.lock();
    workers.append(worker);
    criticalRegion.unlock();
}

void QNetworkInfoPrivate::deleteWorkers( )
{
    for(int i = 0 ;i < workers.size();i++)
    {
        if(workers.at(i))
            delete workers.at(i);
    }
    workers.clear();
    return;
}

int QNetworkInfoPrivate::getChannelAsync( SysInfoProperty channelId , int transId)
{
    QSysInfoElement element;
    int errCode = ErrNoError;

    switch( channelId )
    {
        case Network:
        case SignalStrength:
        case IMSI:
            {
                QAsyncTask* task = new QAsyncTask( transId, channelId );
                QThreadPool::globalInstance()->start( task );
                break;
            }
        case HomeNetwork:
            {
                criticalRegion.lock();
                deleteWorkers();
                criticalRegion.unlock();
                
                QNwDBusWorker* worker = new QNwDBusWorker(channelId,transId);
                QObject::connect(worker,
                        SIGNAL( requestCompleted(int,int,const ChannelInformation&,QNwDBusWorker*)),
                        this,
                        SLOT(handleRequestCompleted (int,int,const ChannelInformation&,QNwDBusWorker*) ) );

                worker->start();
                break;
            }
        default:
            break;
    }
    return errCode;
}

int QNetworkInfoPrivate::getChannel( SysInfoProperty channelId , 
            const ChannelInformation& input ,
            QMap<SysInfoProperty, QVariant>& channelInfo )
{
    int errCode = ErrNoError;

    switch(channelId)
    {
        case Network:
            {
                QString net_name;
                QString net_status;
                QString net_mode;
                QString net_ncode;
                QString net_ccode;
                bool net_locstatus = false;
                qint32 net_lac     = 0;
                qint32 net_cellid  = 0;
                QDBusInterface interface( PHONENET_SERVICE,
                        PHONENET_PATH,
                        PHONENET_IF,
                        QDBusConnection::systemBus());
                if ( true == interface.isValid() )
                {
                    QDBusMessage msg_reg = interface.call( PHONENET_GET_REGISTR_STATUS);
                    if ( msg_reg.type() == QDBusMessage::ReplyMessage &&
                            msg_reg.arguments().count() >= 6 )
                    {
                        net_ncode.setNum(msg_reg.arguments().at(3).toInt());             //Network code
                        net_ccode.setNum(msg_reg.arguments().at(4).toInt());             //Country code
                        net_lac    = msg_reg.arguments().at(1).toInt();                  //Area Code
                        net_cellid = msg_reg.arguments().at(2).toInt();                  //Cell Id
                        net_locstatus = (net_lac == 0 && net_cellid == 0)? false: true;  //Loc status

                        if ( net_ncode.toInt() <= 0 || net_ccode.toInt() <= 0 )
                        {
                            errCode = ErrDataNotFound;
                            channelInfo.insert(channelId,"No Sim");
                            break;
                        }

                        QVariant mode;
                        mode.setValue((uchar)NETWORK_HARDCODED_LATIN_OPER_NAME);

                        QDBusMessage msg_name = interface.call( PHONENET_GET_OPERATOR_NAME, mode, net_ncode.toUInt(),  net_ccode.toUInt());
                        if ( msg_name.type() == QDBusMessage::ReplyMessage &&
                                msg_name.arguments().count() >= 1 &&
                                msg_name.arguments().at(0).type() == QVariant::String )
                        {
                            net_name = msg_name.arguments().at(0).toString();
                        }
                        else
                        {
                            errCode = ErrServiceNotAvailable;
                            break;
                        }

                        qint32  net_type = msg_reg.arguments().at(5).toInt(); //Network type
                        if ( NETWORK_TYPE_HOME == net_type )
                        {
                            net_status = "Current";
                        }
                        else if ( NETWORK_TYPE_PREFERRED == net_type ||
                                NETWORK_TYPE_NOPREF_NOFORBIDDEN == net_type )
                        {
                            net_status = "Available";
                        }
                        else if ( NETWORK_TYPE_FORBIDDEN == net_type )
                        {
                            net_status = "Forbidden";
                        }
                        else
                        {
                            net_status = "Unknown";
                        }
                    }
                    else
                    {
                        errCode = ErrServiceNotAvailable;
                        break;
                    }

                    QDBusMessage msg_mode = interface.call( PHONENET_GET_MODE );
                    if ( msg_mode.type() == QDBusMessage::ReplyMessage &&
                            msg_mode.arguments().count() >= 1 )
                    {
                        qint32  net_type = msg_mode.arguments().at(0).toInt();  //Network mode
                        if ( NETWORK_MODE_GSM == net_type )
                        {
                            net_mode = "ModeGsm";
                        }
                        else if ( NETWORK_MODE_3G == net_type )
                        {
                            net_mode = "Mode3G";
                        }
                        else
                        {
                            net_mode = "ModeCdma";
                        }
                    }
                    else
                    {
                        errCode = ErrServiceNotAvailable;
                        break;
                    }
                }
                else
                {
                    errCode = ErrServiceNotAvailable;
                    break;
                }
                channelInfo.insert(ChannelNetwork::networkName, net_name );
                channelInfo.insert(ChannelNetwork::networkStatus, net_status );
                channelInfo.insert(ChannelNetwork::networkMode, net_mode );
                channelInfo.insert(ChannelNetwork::mobileCountryCode,  net_ccode);
                channelInfo.insert(ChannelNetwork::mobileNetworkCode,  net_ncode);
                channelInfo.insert(ChannelNetwork::locationStatus, net_locstatus);
                channelInfo.insert(ChannelNetwork::areaCode, net_lac );
                channelInfo.insert(ChannelNetwork::cellID, net_cellid );
                break;
            }
        case SignalStrength:
            {
                QDBusInterface interface( PHONENET_SERVICE,
                        PHONENET_PATH,
                        PHONENET_IF,
                        QDBusConnection::systemBus());
                if ( FALSE == interface.isValid() )
                {
                    errCode = ErrServiceNotAvailable; 
                    channelInfo.insert( signalStrength,0 );
                    break;
                }
                QDBusMessage msg = interface.call( PHONENET_GET_SIGNAL_STRENGTH );
                if ( msg.type() == QDBusMessage::ReplyMessage &&
                        msg.arguments().count() >= 1 )
                {
                    qint32 signal_strength = msg.arguments().at(0).toInt();
                    channelInfo.insert( signalStrength,signal_strength );
                }
                else
                {
                    qint32 signal_strength = msg.arguments().at(0).toInt();
                    errCode = ErrServiceNotAvailable; 
                    channelInfo.insert( signalStrength,signalStrength );
                    break;
                }
                break;
            }
        case IMSI:
            {
                QDBusInterface interface( SIM_SERVICE_IMSI,
                        SIM_PATH_IMSI,
                        SIM_IF_IMSI,
                        QDBusConnection::systemBus());

                if ( FALSE == interface.isValid() )
                {
                    channelInfo.insert( imsi,"" );
                    break;
                }

                QDBusMessage msg = interface.call( SIM_GET_IMSI );
                if ( msg.type() == QDBusMessage::ReplyMessage )
                {
                    QString imsi_res = msg.arguments().at(0).toString();
                    channelInfo.insert(imsi,imsi_res);
                }
                else if (msg.type() == QDBusMessage::ErrorMessage)
                {
                    if (msg.errorName()=="com.nokia.csd.SIM.Error.NoSIM")
                    {
                        channelInfo.insert(imsi,QString(""));
                        break;
                    }
                    channelInfo.insert(imsi,QString(""));
                }
                else
                {
                    channelInfo.insert(imsi,QString(""));
                }
                break;
            }
        case CellularNetworkStatus:
            {
                QDBusInterface interface( PHONENET_SERVICE,
                        PHONENET_PATH,
                        PHONENET_IF,
                        QDBusConnection::systemBus());
                if ( FALSE == interface.isValid() )
                {
                    errCode = ErrServiceNotAvailable; 
                    channelInfo.insert( cellularNetworkStatus," " );
                    break;
                }
                QDBusMessage msg = interface.call( PHONENET_GET_REGISTR_STATUS);
                if ( msg.type() == QDBusMessage::ReplyMessage &&
                        msg.arguments().count() >= 1 )
                {
                    qint32 reg_status = msg.arguments().at(0).toInt();
                    if ( NETWORK_REG_STATUS_HOME == reg_status )
                    {
                        channelInfo.insert(  cellularNetworkStatus, "RegisteredOnHomeNetwork" );
                    }
                    else if ( NETWORK_REG_STATUS_ROAM == reg_status ||
                            NETWORK_REG_STATUS_ROAM_BLINK == reg_status)
                    {
                        channelInfo.insert(  cellularNetworkStatus, "RegisteredRoaming" );
                    }
                    else if ( NETWORK_REG_STATUS_NOSERV == reg_status ||
                            NETWORK_REG_STATUS_NOSERV_NOTSEARCHING == reg_status ||
                            NETWORK_REG_STATUS_NOSERV_SIM_REJECTED_BY_NW == reg_status)
                    {
                        channelInfo.insert(  cellularNetworkStatus, "NotRegisteredNoService");
                    }
                    else if ( NETWORK_REG_STATUS_NOSERV_SEARCHING == reg_status )
                    {
                        channelInfo.insert(  cellularNetworkStatus, "NotRegisteredSearching");
                    }
                    else if ( NETWORK_REG_STATUS_NOSERV_NOSIM == reg_status )
                    {
                        channelInfo.insert(  cellularNetworkStatus, "NotRegisteredEmergencyOnly");
                    }
                    else if ( NETWORK_REG_STATUS_POWER_OFF == reg_status ||
                            NETWORK_REG_STATUS_NSPS == reg_status ||
                            NETWORK_REG_STATUS_NSPS_NO_COVERAGE == reg_status )
                    {
                        channelInfo.insert(  cellularNetworkStatus, "RegistrationDenied");
                    }
                    else
                    {
                        channelInfo.insert(  cellularNetworkStatus, "RegistrationUnknown");
                    }
                }
                else
                {
                    errCode = ErrServiceNotAvailable; 
                    channelInfo.insert( cellularNetworkStatus," " );
                    break;
                }
                break;
            }
        case WirelessConnSupport:
            {
                QStringList AvailableWIInterfaces;

                QDBusInterface interface( WCS_SERVICE,
                        WCS_PATH,
                        WCS_IF,
                        QDBusConnection::systemBus());
                if (TRUE == interface.isValid())
                {
                    QDBusReply<QStringList> reply = interface.call(WCS_GET);
                    if(TRUE == reply.isValid())
                    {
                        channelInfo.insert(WirelessConnSupport,reply.value());
                        break;
                    }
                }

                QDBusInterface interface_hal( HAL_WCS_SERVICE,
                        HAL_WCS_PATH,
                        HAL_WCS_IF,
                        QDBusConnection::systemBus());

                if ( FALSE == interface_hal.isValid() )
                {
                    errCode = ErrServiceNotAvailable;
                    break;
                }

                QDBusMessage wi_wlan = interface_hal.call( QDBus::Block,
                        HAL_WCS_FIND_DEVICE,
                        HAL_WI_WLAN );
                if ( wi_wlan.type() == QDBusMessage::ReplyMessage &&
                        wi_wlan.arguments().count() >= 1 &&
                        wi_wlan.arguments().at(0).type() == QVariant::StringList )
                {
                    QStringList list_key = wi_wlan.arguments().at(0).toStringList();
                    if ( FALSE == list_key.isEmpty())
                    {
                        AvailableWIInterfaces.append(WI_NAME_WLAN);
                    }
                }

                QDBusInterface interface_bluez( BLUEZ_SERVICE,
                        BLUEZ_PATH,
                        BLUEZ_IF_MANAGER,
                        QDBusConnection::systemBus());
                if ( FALSE == interface_bluez.isValid() )
                {
                    errCode = ErrServiceNotAvailable;
                    break;
                }
                QDBusMessage wi_bt = interface_bluez.call( BLUEZ_GET_PATH );
                if ( wi_bt.type() == QDBusMessage::ReplyMessage )
                {
                    QString path = wi_bt.arguments().at(0).value<QDBusObjectPath>().path();
                    if (FALSE == path.isEmpty())
                    {
                        AvailableWIInterfaces.append(WI_NAME_BLUETOOTH);
                    }

                }
                else
                {
                    errCode = ErrServiceNotAvailable;
                    break;
                }
                AvailableWIInterfaces.append(WI_NAME_GSM);
                AvailableWIInterfaces.append(WI_NAME_WCDMA);
                channelInfo.insert(connectionTypes,AvailableWIInterfaces);
                break;
            }
        case ActiveDataConnections:
            {
                QList< QMap<SysInfoProperty,QVariant> > activeConnections;
                QNetworkConfigurationManager manager;
                QList<QNetworkConfiguration> activeConfigurations = manager.allConfigurations (QNetworkConfiguration::Discovered);

                int size =  activeConfigurations.count();
                QVariant qVar;
                if(size>0)
                {
                    for (qint32 i=0; i<size; i++)
                    {
                        QMap<SysInfoProperty,QVariant> data;
                        data.insert( connectionType ,activeConfigurations.at(i).bearerName() );
                        data.insert( connectionName ,activeConfigurations.at(i).name() );
                        activeConnections.append(data );
                    }
                    qVar.setValue(activeConnections);
                    channelInfo.insert(dataConnections, qVar);
                }
                else
                {
                    errCode =  ErrDataNotFound;
                }
                break; 
            }
        default:
            errCode = ErrNotSupported;
            //TODO:errData.insert( channelId, "" );
            break;
    }
    return errCode;
}
void QNetworkInfoPrivate::startSignalStrengthChannelCb( const QDBusMessage & message )
{
    int errCode = ErrNoError;
    if ( TRUE == triggerMap.contains( SignalStrength ) )
    {
        ChannelInformation trigger = triggerMap.value( SignalStrength );
        qint32 trId = trigger.value( signalStrength, -1 ).toInt(); //defaul val = - 1

        qint32 signal_strength = message.arguments().at(0).toInt();

        if ( trId == -1 || signal_strength < trId  )
        {
            ChannelInformation  channelInfo;
            channelInfo.insert( signalStrength,  signal_strength );
            emit requestStarted(errCode ,SignalStrength , channelInfo); 
        }
    }
}

void QNetworkInfoPrivate::startNetworkChannelCb( const QDBusMessage & message )
{
    int errCode = ErrNoError;

    if ( TRUE == triggerMap.contains( Network ) )
    {
        QMap<SysInfoProperty, QVariant> channelInfo;
        QMap<SysInfoProperty, QVariant> input;

        errCode = getChannel(Network,input,channelInfo);

        if ( (errCode != ErrNoError) || channelInfo.isEmpty())
        {
            errCode = ErrDataNotFound;
        }
        emit requestStarted(errCode ,Network , channelInfo); 
    }
    return;
}

int QNetworkInfoPrivate::startChannel( SysInfoProperty channelId,
        const ChannelInformation& triggerInfo,
        int transId)
{
    int errCode = ErrNoError;

    switch(channelId)
    {
        case SignalStrength:
            {
                if ( FALSE == QDBusConnection::systemBus().connect( PHONENET_SERVICE,
                            PHONENET_PATH,
                            PHONENET_IF,
                            PHONENET_CHANGE_SIGNAL_STRENGTH,
                            this,
                            SLOT(startSignalStrengthChannelCb(const QDBusMessage& ))) )
                {
                    errCode = ErrServiceNotAvailable;
                    break;
                }
                else
                    triggerMap.insert(channelId,triggerInfo);
                break;
            }
        case Network:
            {
                if ( FALSE == QDBusConnection::systemBus().connect( PHONENET_SERVICE,
                            PHONENET_PATH,
                            PHONENET_IF,
                            PHONENET_REGISTR_STATUS_CHANGED,
                            this,
                            SLOT(startNetworkChannelCb(const QDBusMessage& ))) )
                {
                    errCode = ErrServiceNotAvailable;
                    break;
                }
                else
                    triggerMap.insert(channelId,triggerInfo);
                break;
            }
        default:
            errCode = QNetworkInfo::startChannel(channelId, triggerInfo, transId);
            break;
    }
    return errCode;
}

int QNetworkInfoPrivate::stopChannel( SysInfoProperty channelId ) 
{
    int errCode = ErrNoError;

    switch( channelId )
    {
        case SignalStrength:
            {
                QDBusConnection::systemBus().disconnect( PHONENET_SERVICE,
                        PHONENET_PATH,
                        PHONENET_IF,
                        PHONENET_CHANGE_SIGNAL_STRENGTH,
                        this,
                        SLOT(startSignalStrengthChannelCb(const QDBusMessage& )));
                triggerMap.remove(channelId);
                break;
            }
        case Network:
            {
                QDBusConnection::systemBus().disconnect( PHONENET_SERVICE,
                        PHONENET_PATH,
                        PHONENET_IF,
                        PHONENET_REGISTR_STATUS_CHANGED,
                        this,
                        SLOT(startNetworkChannelCb(const QDBusMessage& )));
                triggerMap.remove(channelId);
                break;
            }
        default:
            errCode = QNetworkInfo::stopChannel(channelId);
            break;
    }
    return errCode;
}

int QNetworkInfoPrivate::setChannelSync( SysInfoProperty channelId , const QVariant& input,QMap<SysInfoProperty, QVariant>& errData)
{
    int errCode = ErrNoError;

    switch(channelId)
    {
        default:
            
            errCode = QNetworkInfo::setChannelSync(channelId, input, errData);
            break;
    }
    return errCode;
}

int QNetworkInfoPrivate::cancel( SysInfoProperty channelId , int transId)
{
    int errCode = ErrNoError;
    
    switch( channelId )
    {
        case HomeNetwork:
            {
/*                QDBusConnection::systemBus().disconnect(CSNET_SERVICE, CSNET_PATH, CSNET_NET_REGISTRATION_IF, "AvailableNetworks",
                        this, SLOT(asyncHomeNetworkChannelCb(const QDBusMessage&)));
                QDBusConnection::systemBus().disconnect(CSNET_SERVICE, CSNET_PATH, CSNET_NET_REGISTRATION_IF, "QueryFailed",
                        this, SLOT(asyncHomeNetworkChannelCb(const QDBusMessage&)));
                        */
                break;
            }
        default:
            errCode = QNetworkInfo::cancel(channelId , transId);
            break;
    }

    return errCode;
}

int QNetworkInfoPrivate::getChannelList(QList<SysInfoProperty>& channelList) 
{
    channelList += Network;
    channelList += HomeNetwork;
    channelList += SignalStrength;
    channelList += IMSI;                    
    channelList += CellularNetworkStatus;
    channelList += WirelessConnSupport;
    channelList += ActiveDataConnections;
    
    return 0;
}
