#include "qnwdbusworker.h"
#include "qsysinfoerrors.h"
#include "qisysinfo.h"
#include "qsysinfofactory.h"
#include <QDBusMessage>
#include <QDBusInterface>
#include <QDBusArgument>
#include <QDBusConnection>
#include <QDBusReply>
#include "qsysinfoproperties.h"

using namespace NetworkInfo;
using namespace NetworkInfo::ChannelHomeNetwork;

//Home Network
#define HN_SERVICE           "com.nokia.phone.SIM"
#define HN_PATH              "/com/nokia/phone/SIM"
#define HN_IF                "Phone.Sim"
#define HN_GET_OPERATOR_NAME "get_service_provider_name"
#define HN_GET_SERVICE_INFO  "read_hplmn"

struct AvailableConnections
{
    QString var0;
    QString var1;
    QString var2;
    QString var3;
    QString var4;
    QString var5;
    bool var6;
};
    Q_DECLARE_METATYPE(AvailableConnections)
const QDBusArgument &operator>>(const QDBusArgument &argument, AvailableConnections &mystruct)
{
    argument.beginStructure();
    argument >> mystruct.var0 >> mystruct.var1 >> mystruct.var2 >> mystruct.var3 >> mystruct.var4 >> mystruct.var5 >> mystruct.var6;
    argument.endStructure();
    return argument;
}

QNwDBusWorker::~QNwDBusWorker()
{
}

int QNwDBusWorker::start()
{
    int errCode = ErrNoError;
    QDBusInterface interface( HN_SERVICE,
            HN_PATH,
            HN_IF,
            QDBusConnection::systemBus());
    if ( true == interface.isValid() )
    {
        if ( FALSE == interface.callWithCallback( HN_GET_OPERATOR_NAME,
                QList<QVariant>(),
                this,
                SLOT(asyncHomeNetworkChannelCb(const QDBusMessage&)) ))
        {
            errCode = ErrDataNotFound;
        }
    }
    else
        errCode = ErrDataNotFound;

    return errCode;
}

QNwDBusWorker::QNwDBusWorker(SysInfoProperty cId,int transId)
{
    transactionId = transId;
    channelId = cId;
}

void QNwDBusWorker::asyncHomeNetworkChannelCb(const QDBusMessage& message)
{
    int errCode = ErrNoError;
    QMap<SysInfoProperty, QVariant> channelInfo;
    if ( message.type() == QDBusMessage::ReplyMessage )
    {
        QMap<SysInfoProperty, QVariant> input;
        QISysInfo* instance = NULL; 

        QSysInfoFactory& factory =  QSysInfoFactory::factoryInstance();    
        instance =  factory.getSysInfoInstance( HomeNetwork ) ; 
        if( instance )
        {
            errCode = instance->getChannel( NetworkInfo::ChannelNetwork::Network, input , channelInfo );


            if ( (errCode != ErrNoError) || channelInfo.isEmpty())
            {
                emit requestCompleted(ErrServiceNotAvailable,transactionId,channelInfo,this) ;
                return;
            }

            channelInfo.insert( networkName, message.arguments().at(0).toString() );

            QDBusInterface interface( HN_SERVICE,
                    HN_PATH,
                    HN_IF,
                    QDBusConnection::systemBus());
            if (FALSE == interface.isValid())
            {
                emit requestCompleted(ErrServiceNotAvailable,transactionId,channelInfo,this) ;
                return;
            }
            QDBusReply<QByteArray> reply = interface.call(HN_GET_SERVICE_INFO);
            // The MNC and MCC are split into Hex numbers in the received byte array.
            // The MNC can be 2 or 3 digits long. If it is 2 digits long, it ends with 0xF.
            // The order of the Hex numbers in the reply is:
            // mcc2 mcc1 mnc3 mcc3 mnc2 mnc1
            QString mnc;
            QString mcc;
            if (TRUE == reply.isValid())
            {
                QString temp = reply.value().toHex();
                if(temp.count() == 6)
                {
                    //MNC//                                     

                    mnc.insert(0,temp.at(5));
                    mnc.insert(1,temp.at(4));
                    if(temp.at(2) != 'f')
                    {
                        mnc.insert(2,temp.at(2));
                    }
                    //MCC
                    mcc.insert(0,temp.at(1));
                    mcc.insert(1,temp.at(0));
                    mcc.insert(2,temp.at(3));
                }
            }
            else
            {
                emit requestCompleted(ErrServiceNotAvailable,transactionId,channelInfo,this) ;
                return;
            }
            channelInfo.insert( mobileCountryCode, mcc );
            channelInfo.insert( mobileNetworkCode, mnc );
            channelInfo.insert( locationStatus, (mnc.toInt() == 0 && mcc.toInt() == 0) ? false:true );

            emit requestCompleted(errCode,transactionId,channelInfo,this) ;
        }
        else
        {
            //no instance found
            emit requestCompleted(ErrServiceNotAvailable,transactionId,channelInfo,this) ;
        }
    }   
}

void QNwDBusWorker::asyncHomeNetworkChannelErrCb(const QDBusMessage& message)
{
    ChannelInformation infoList;
    infoList.insert(HomeNetwork,"Error callback was called");

    emit requestCompleted(ErrServiceNotAvailable,transactionId,infoList,this) ;
}

