#include "qsysconfig_p.h"
#include "qsysinfoproperties.h"
#include "qsysinfotypes.h"
#include <QDBusMessage>
#include <QDBusReply>
#include <QDBusInterface>
#include <QDBusConnection>
#include <QFile>
#include <QStringList>

//Active Profile
#define PROFILED_SERVICE   "com.nokia.profiled"
#define PROFILED_PATH   "/com/nokia/profiled"
#define PROFILED_INTERFACE   "com.nokia.profiled"
#define GET_PROFILE   "get_profile"  //get active profile
#define GET_VALUE "get_value"
#define GET_VIBRATION "vibrating.alert.enabled"
#define GET_R_VOLUME "ringing.alert.volume"
#define GET_M_VOLUME "sms.alert.volume"

#define PROFILED_KEY_MSG_VOL "sms.alert.volume"
#define PROFILED_KEY_CALL_VOL "ringing.alert.volume"
#define PROFILED_KEY_VIBRA_ON  "vibrating.alert.enabled"
#define PROFILED_GET_VALUE   "get_value"

using namespace SysConfig ;
using namespace SysConfig::ChannelActiveProfile ;
using namespace SysConfig::ChannelprocessorUtilizationPercent ;

QSysConfigPrivate::QSysConfigPrivate()
{
}

int QSysConfigPrivate::getChannel( SysInfoProperty channelId , 
        const ChannelInformation& input ,
        QMap<SysInfoProperty, QVariant>& channelInfo )
{
    int errCode = ErrNoError;

    switch( channelId )
    {
        case ActiveProfile:
            {
                bool vibration = FALSE;
                qint32 r_vol = 0;
                qint32 m_vol = 0;

                QDBusInterface interface( PROFILED_SERVICE,
                        PROFILED_PATH,
                        PROFILED_INTERFACE,
                        QDBusConnection::sessionBus() );
                if ( FALSE == interface.isValid() )
                {
                    errCode = ErrServiceNotAvailable;
                    break;
                }

                QDBusReply<QString> profileNameReply = interface.call( GET_PROFILE );
                if ( TRUE == profileNameReply.isValid() )
                {
                    const QString profileName = profileNameReply.value();

                    QDBusReply<QString> vibratingAlertEnabledReply = interface.call( GET_VALUE , profileName, GET_VIBRATION );
                    if ( TRUE == vibratingAlertEnabledReply.isValid() )
                    {
                        vibration = vibratingAlertEnabledReply.value() == "On";
                    }
                    else
                    {
                        errCode = ErrServiceNotAvailable;
                        break;
                    }

                    QDBusReply<QString> ringingAlertVolumeReply = interface.call( GET_VALUE, profileName, GET_R_VOLUME );
                    if ( TRUE == ringingAlertVolumeReply.isValid() )
                    {
                        r_vol = ringingAlertVolumeReply.value().toInt();
                    }
                    else
                    {
                        errCode = ErrServiceNotAvailable;
                        break;
                    }

                    QDBusReply<QString> msgAlertVolumeReply = interface.call( GET_VALUE, profileName, GET_M_VOLUME );
                    if ( TRUE == msgAlertVolumeReply.isValid() )
                    {
                        m_vol = msgAlertVolumeReply.value().toInt();
                    }
                    else
                    {
                        errCode = ErrServiceNotAvailable;
                        break;
                    }
                }
                else
                {
                    errCode = ErrServiceNotAvailable;
                    break;
                }
                channelInfo.insert( msgRingtoneVolume, m_vol );
                channelInfo.insert( ringtoneVolume, r_vol );
                channelInfo.insert( vibrationActive, vibration );
                break;
            }
#if 0
        case processorUtilizationPercent:
            {
                
                QFile file(QString("/proc/stat"));
                if (!file.open(QFile::ReadOnly))
                {
                    errCode = -1 ;
                    //TODO : "Error open file";
                    break;
                }
                QByteArray lineSrc=file.readLine();
                QString line(lineSrc);
                QStringList strippedLine=line.split(QChar(' '));

                quint32 user=strippedLine.at(2).toUInt();
                quint32 nice=strippedLine.at(3).toUInt();
                quint32 system=strippedLine.at(4).toUInt();
                quint32 idle=strippedLine.at(5).toUInt();
                lineSrc.clear();
                line.clear();
                strippedLine.clear();
                file.close();

                usleep(1000000);

                if (!file.open(QFile::ReadOnly))
                {
                    errCode = -1 ;
                    //TODO : "Error open file";
                    break;
                }

                lineSrc=file.readLine();
                line=lineSrc;
                strippedLine=line.split(QChar(' '));

                quint32 user2=strippedLine.at(2).toUInt();
                quint32 nice2=strippedLine.at(3).toUInt();
                quint32 system2=strippedLine.at(4).toUInt();
                quint32 idle2=strippedLine.at(5).toUInt();
                lineSrc.clear();
                line.clear();
                strippedLine.clear();
                file.close();

                float percent=100-(((float)(idle2-idle))/((float)((user2+nice2+system2+idle2)-(user+nice+system+idle)))*100);
                channelInfo.insert(percent, qRound(percent));
                break;
            }
#endif
        default:
            {
                errCode = getChannel (channelId , input , channelInfo );
                break;
            }
    }
    return errCode;
}

int QSysConfigPrivate::getChannelAsync( SysInfoProperty channelId , int transId)
{
    int errCode = ErrNoError;

    switch(channelId)
    {
        default:
            errCode =  QSysConfig::getChannelAsync( channelId, transId);
    }
    return errCode;
}

int QSysConfigPrivate::startChannel( SysInfoProperty channelId,
        const ChannelInformation& triggerInfo,
        int transId)
{
    int errCode = ErrNoError;

    switch(channelId)
    {
        default:
            errCode =  QSysConfig::startChannel( channelId,triggerInfo , transId);
    }
    return errCode;
}

int QSysConfigPrivate::stopChannel( SysInfoProperty channelId)
{
    int errCode = ErrNoError;

    switch(channelId)
    {
        default:
            errCode =  QSysConfig::stopChannel( channelId );
    }
    return errCode;
}

int QSysConfigPrivate::setChannelSync( SysInfoProperty channelId ,  const QVariant& input, QMap<SysInfoProperty, QVariant>& errData)
{
    int errCode = ErrNoError;

    switch( channelId )
    {
        default:
            errCode =  QSysConfig::setChannelSync( channelId , input, errData);
            break;
    }
    return errCode;
}
int QSysConfigPrivate::cancel( SysInfoProperty channelId , int transId)
{
    int errCode = ErrNoError;

    switch( channelId )
    {
        default:
            errCode =  QSysConfig::cancel( channelId , transId);
            break;
    }
    return errCode;
}

int QSysConfigPrivate::getChannelList(QList<SysInfoProperty>& channelList) 
{
    channelList += ActiveProfile;
    return 0;
}
