#include "qsysinfofactory.h"
#define __ENABLE_PROPERTIES__
#include "qsysinfoproperties.h"
#include "qdeviceinfo_p.h"
#include "qdeviceextensions_p.h"
#include "qnetworkinfo_p.h"
#include "qmultimedia_p.h"
#include "qmedia_p.h"
#include "qsysconfig_p.h"

QSysInfoFactory::QSysInfoFactory()
{
    sysInfoInstances = new QISysInfo*[groupCount];
    for( int i=0; i < groupCount-1; i++ )
    {
        sysInfoInstances[i] = NULL; 
    }
}

QSysInfoFactory::~QSysInfoFactory()
{
    for( int i=0; i < groupCount-1; i++ )
    {
        if( sysInfoInstances[i] )
            delete sysInfoInstances[i];
    }
    delete sysInfoInstances;
    
}

QList<QISysInfo*> QSysInfoFactory::getSysInfoInstances()
{
    QList<QISysInfo*> list; 
    for( int i=1; i<= groupCount; i++) {
        list.append( getSysInfoInstance( i << GroupShiftBits ) );
    }
    return list;
}

QISysInfo* QSysInfoFactory::getSysInfoInstance( SysInfoProperty channelId )
{
    QISysInfo* inst = NULL;
    if (isChannelValid (channelId))
    {
        int groupId = channelId >> GroupShiftBits;
        int index = groupId - 1;

        switch( groupId ) 
        {
            case DeviceInfo::groupId:
                if( !sysInfoInstances[ index ] )
                    sysInfoInstances[ index ] = new QDeviceInfoPrivate();
                inst = sysInfoInstances[ index ];
                break;
            case DeviceExtensions::groupId:
                if( !sysInfoInstances[ index ] )
                    sysInfoInstances[ index ]  = new QDeviceExtensionsPrivate();
                inst = sysInfoInstances[ index ];
                break;
            case NetworkInfo::groupId:
                if( !sysInfoInstances[ index ] )
                    sysInfoInstances[ index ] = new QNetworkInfoPrivate();
                inst = sysInfoInstances[ index ];
                break;
            case Multimedia::groupId:
                if( !sysInfoInstances[ index ] )
                    sysInfoInstances[ index ] = new QMultimediaPrivate();
                inst = sysInfoInstances[ index ];
                break;
            case Media::groupId:
                if( !sysInfoInstances[ index ] )
                    sysInfoInstances[ index ] = new QMediaPrivate();
                inst = sysInfoInstances[ index ];
                break;
            case SysConfig::groupId:
                if( !sysInfoInstances[ index ] )
                    sysInfoInstances[ index ] = new QSysConfigPrivate();
                inst = sysInfoInstances[ index ];
                break;
            default:
                break;
        }
    }
    return inst;
}

QSysInfoFactory& QSysInfoFactory::factoryInstance()
{
    static QSysInfoFactory instance;
    return instance;
}

bool QSysInfoFactory::isChannelValid(SysInfoProperty channelId)
{
   int groupChannel = channelId >>  ChannelShiftBits ;
   if (groupChannel)
   {
       if ( (groupChannel << ChannelShiftBits) ^ channelId)
           return false;
       else
           return true;
   }
   else
       return false;

}


