/*
* ============================================================================
*  Name        : qsysinfoservice.h
*  Part of     : serviceproviders / WRT
*  Description : Qt class for sysinfo service
*  Version     : %version: 1 % << Don't touch! Updated by Synergy at check-out.
*
 * Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
 *
 * This file is part of Qt Web Runtime.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef QSYSINFOSERVICE_H_
#define QSYSINFOSERVICE_H_

#include <QtDBus>
#include <QAbstractSecuritySession>
#include "secsession.h"
#include "qsysinfoelement.h"

static const char* KISysInfoId = "com.nokia.ISysInfo";
typedef ChannelInformation ChannelInfoMap;


class SysInfoService : public QObject//,
{
    Q_OBJECT

public:
    //public interfaces:
    SysInfoService(QObject* parent);
    virtual ~SysInfoService();
    /**
    * Provides implementation for setSecuritySession derived from
    * IServiceSecurity.
    * Needed to handle capabilities.
    * @since x.y
    * @param aSecSession - Security Manager session
    * @return none
    */
    void setSecuritySession( QtMobility::QAbstractSecuritySession *aSecSession );

signals:
    void asyncCallback( int, int, const QMap<QString,QVariant>& );
    void asyncCallbackStChannel( int aError, int aTransId, const QMap<QString,QVariant>& );

public slots:

    void handleRequestCompleted(int, int,const ChannelInformation&);
    void handleRequestStarted(int, int,const ChannelInformation&);

    //Slots
    QVariant getChannelList();
    QVariant setChannelSync( const QString& aChannel, const QVariant& aInputData);
    QVariant getChannelSync( const QString& aChannel , const QMap<QString,QVariant>& aInputData );
    QVariant getChannelSync( const QString& aChannel );
    QVariant getChannelAsync( const QString& aChannel, const int transId );
    QVariant cancel( const int aTransId );
    QVariant startChannel( const QString& aChannel, const int transId, const QVariant& aTrigger );
    QVariant stopChannel( const QString& aChannel );

private:
    QString getErrorMessage( int err, const QMap<SysInfoProperty, QVariant>& errData );

    int fromProperties( const QMap<SysInfoProperty, QVariant>& channelInfo, QMap<QString,QVariant>& channelInfoData );

    int toProperties( const QMap<QString, QVariant>& channelInfoData, QMap<SysInfoProperty, QVariant>& channelInfo );

private:
    //Private members
    /**
     *  Handle to security session
     */
    QtMobility::QAbstractSecuritySession *_sec_session;
            
};


#endif //QSYSINFOSERVICE_H_
