use strict;
use warnings;
use FindBin qw($Bin);
use Getopt::Long;
use File::Spec;
use IPC::Open3;

my $usage = "$0 -srcroot <path to wrt source> -template <template file> -output <output file> -default \"<default value>\"\n\n";

GetOptions(
    'srcroot=s'   => \my $srcroot,
    'template=s'  => \my $template,
    'output=s'    => \my $output,
    'default=s'   => \my $default,
    'help'        => \my $help,
  )
  or die $usage;
($help or !$srcroot or !$template or !$output or !$default) and die $usage;

$srcroot = File::Spec->rel2abs($srcroot);
$template = File::Spec->rel2abs($template);
$output = File::Spec->rel2abs($output);

my $svnrevision = "";

chdir($srcroot) or die("The given srcroot is invalid.");

my $SVN_ENTRIES_PATH = File::Spec->canonpath(".svn/entries");
my $SVN_URL = "https://trace1.isource-nokia.nokia.com/isource/svnroot/BrDo/cwrt/trunk";

sub get_rev_from_svn_entries
{
    my $res = "";

    open(FP, $SVN_ENTRIES_PATH) or return $res;
    my @lines = <FP>;
    close(FP);

    my $url_line = $lines[4];
    chomp($url_line);

    if ($url_line eq $SVN_URL) {
        $res = int($lines[3]);
    }
    return $res;
}

local (*IN, *OUT, *ERR);
my $p = open3(*IN, *OUT, *ERR, "svn info $srcroot");
close(IN);
close(ERR);
waitpid($p, 0);

# get svn version info
if ($? == 0) {
    while (<OUT>) {
        if (/Revision: (\d+)/) {
            $svnrevision = $1;
        }
    }
} elsif (-e $SVN_ENTRIES_PATH) {
    $svnrevision = get_rev_from_svn_entries();
} else {
    if (-d ".git") {
        warn "Git is not being handled yet.\n";
    } else {
        warn "This directory is not under svn.\n";
    }
}

close(OUT);

$svnrevision =~ /^[+-]?\d+$/ or $svnrevision = $default;

print "Setting the revision to $svnrevision.";

open tmpl_file, "< $template" or die $!;
open out_file, "> $output" or die $!;

while (<tmpl_file>) {
    s/#define\s+WRT_VERSION_SVN\s+\"\w+\"/#define WRT_VERSION_SVN \"$svnrevision\"/g;
    print out_file $_;
}

close tmpl_file;
close out_file;
