/*
 * Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
 *
 * This file is part of Qt Web Runtime.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef DIGSIGSERVICE_H_
#define DIGSIGSERVICE_H_

#include <QtCore/QObject>
#include <QString>
#include <QStringList>

enum WidgetValidationError {
    ValidateOK = 0,
    FindSignatureFailed = 3,
    SignatureParseFailed = 4,
    SignatureElementsMissing = 5,
    ReferencesMissing = 6,
    ReferenceValidationFailed = 7,
    CertMappingFailed,
    RootCertificateNotFound,
    SelfSignedRootCertNotFound,
    CertificateDatesInvalid,
    CertificateExtensionsInvalid,
    ChainValidationFailed,
    SignatureDecryptionFailed,
    SignatureDigestInfoInvalid,
    SignedInfoCanonicalizationFailed,
    SignedInfoDigestFailed,
    SignedInfoDigestMatchFailed
};

class DigSigService : public QObject {

public:
    DigSigService();
    ~DigSigService();

    WidgetValidationError validateWidget(QString path, bool cleanup = false);
    QString getCertificateAKI();
    QStringList getCertificateAKIs();

private:
    bool findDigitalSignatures(QStringList& digSig, const QString& path);
    bool rmDir(const QString& path);

    QString m_validCertificateAKI;
    QStringList m_certificateAKIs;

};

#endif /* DIGSIGSERVICE_H_ */
