/*
 * Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
 *
 * This file is part of Qt Web Runtime.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef WEBWIDGET_H_
#define WEBWIDGET_H_

#include <QString>
#include <QStringList>
#include <QObject>
#include <QSize>
#include <QMap>
#include "WebAppRegistry.h"
#include "widgetcoreglobal.h"

class WgtWidget;
class WidgetProperties;


namespace WRT {
namespace Maemo {
class DesktopFileReader;

/** Represents one web widget.
 This class represents single web widget that is shown
 on the screen. Class encapsulates metadata that is used
 when rendering the widget.
 */
class WIDGETCORE_EXPORT WebWidget: public QObject {
        Q_OBJECT
    public:

        /**
         * WebWidget Type
         */

        enum WebWidgetErrorCode {
             NoError = 0,
             GeneralError,
             MetadataError,
             DesktopFileError,
             RegistrationError,
             CapabilityValidationError
         };


        /** Overloaded widget constructor.
         * Widget constructor taking desktop file path as an input.
         * Widget language is (in the order of preference) system language
         * from LANG environment variable, WebWidget default language or English.
         * @param path Absolute path to widget's desktop file.
         */
        WebWidget(const WebAppInfo* webAppInfo,
                  QObject* parent = 0);

        virtual ~WebWidget();

        /** Checks if the widget is valid. This function should be called after
         *  creating a new WebWidget.
         *  @return false, if widget is invalid.
         */
        bool isValid();

        /** sets the desktop icon
         * @param uri fullpath to icon
         * @param width desired width of icon
         * @param height, desired height if icon
         *
         */

    public:

        QString getName() const;
        QUrl getUrl() const;
        QString getWidgetResource() const;

        WebWidgetErrorCode errorCode();

        /** Gets the value of X-MeeGoApplet-WRT-Protected-Store from the .desktop file.
         * @return String representation of the key or an empty string if not found.
         */
        QString protectedStorageName() const;

        // FIXME: Remove this function and use widget registry
        // to get this info
        WidgetProperties* widgetProperties();

        /** Gets the widget size.
         * Gets the size of this widget.
         * @return Size of this widget.
         */
        QSize size() const;

        QStringList supportedViewModes() const;

    private:
        // methods
        void init(const DesktopFileReader & metadata);
        bool readConfigfile();

    private:
        // data
        const WebAppInfo* m_webAppInfo;
        QMap<QString, QVariant> m_properties;

        QString m_locale;
        QString m_webAppId;
        QString m_widgetRoot;

        bool m_valid;
        WgtWidget * m_widget;
        WebWidgetErrorCode m_errorCode;
};
}
}

#endif /* WEBWIDGET_H_ */
