/*
 * Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
 *
 * This file is part of Qt Web Runtime.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "jsescapebutton.h"

#include <qwebframe.h>
#include <qwebpage.h>

#include <QDebug>


namespace WRT {


/*!
    \class WRT::JSEscapeButton
    \brief The WRT::JSEscapeButton

    \inmodule WidgetCore

    WRT::JSEscapeButton evaluates escapebutton.js JavaScript file.
*/

/*!
    Constructs an instance of JSEscapeButton that evaluates escapebutton.js resource. Native part is exposed to escapeButtonCPP.
*/
JSEscapeButton::JSEscapeButton()
    :JSResourceWrapper("escapebutton.js", "escapeButtonCPP")
    , m_mode("")
{
}

JSEscapeButton::~JSEscapeButton()
{
}

/*!
    Gets \a mode of the escape button.
*/
QString JSEscapeButton::mode()  const
{
    return m_mode;
}

/*!
    Sets "exit" or "back" as \a mode of the escape button.
*/
void JSEscapeButton::setMode(QString mode)
{
    QStringList escapeButtonModes;
    escapeButtonModes << "exit" << "back";

    if (escapeButtonModes.contains(mode, Qt::CaseInsensitive)) {
        m_mode = mode;
        emit escapeModeChanged(mode);
    }
}

/*!
 *    Method for firing onselect callback of the escape button.
 */
void JSEscapeButton::escapeButtonClicked() {
    QString script = QString("if (typeof(window.nokia.device.escapeButton.onclick) === 'function') { window.nokia.device.escapeButton.onclick(); }");
    JSResourceWrapper::webPage()->mainFrame()->evaluateJavaScript(script);
}

void JSEscapeButton::pageOnReload()
{
    setMode(QString("exit"));
    JSResourceWrapper::pageOnReload();
}

}
