/*
 * Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
 *
 * This file is part of Qt Web Runtime.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


#ifndef __WEBJSMENUITEM_H
#define __WEBJSMENUITEM_H

#include "jsresourcewrapper.h"

#include <QVariantMap>

namespace WRT {

class Action;

class JSMenuItem : public JSResourceWrapper
{
    Q_OBJECT
    Q_PROPERTY(QString name READ text WRITE setText)
    Q_PROPERTY(QString icon READ icon WRITE setIcon)
    Q_PROPERTY(bool disabled READ isDisabled WRITE setDisabled)

public:

    explicit JSMenuItem(QString exposeName);
    virtual ~JSMenuItem();

    QString text() const;
    void setText(QString name);
    QString icon() const;
    void setIcon(QString icon);
    bool isDisabled() const;
    void setDisabled(bool disabled);
    void setAppPath(QString path);
    void setWidgetId(QString widgetId);

    Action* action();

public slots:
    QVariantMap isActivated();
    void setActivated(bool activated);
    void addToJavaScriptWindowObject();
    bool addMenuItem(int nativeInstanceHandle);
    bool removeMenuItem(int nativeInstanceHandle);
    bool insertMenuItem(int nativeInstanceHandle, int beforeIndex);

private:
    Action* m_action;
    QString m_icon;
    QString m_appPath;
    QString m_widgetId;
};

}

#endif
