/*
 * Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
 *
 * This file is part of Qt Web Runtime.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef WRTAPIPROVIDER_HH
#define WRTAPIPROVIDER_HH

#include <QStringList>
#include <QMap>

#include "serviceprovider.h"
#include "wrtapiinterface.h"
#include "wrtglobal.h"

namespace WRT {

    class ServiceProxy;
    class WrtPage;
    class JSResourceWrapper;
    
    class WrtApiProvider;

    class WrtApiProviderPrivate
    {

    public:
        WrtApiProviderPrivate(WrtApiProvider*);
        ~WrtApiProviderPrivate();

        /**
         * Adds JavaScript API to map of APIs and to service loader. In addition, adds web page context to api object.
         */
        void finalizeApi(JSResourceWrapper* api, JSApi::WidgetJavaScriptApiFlag jsApi);

    public:
        QMap<QString, ServiceProxy*> m_services;
        WrtPage* m_page;
        JSApi::WidgetJavaScriptApiFlags m_apiNames;
        QMap<JSApi::WidgetJavaScriptApiFlag, WrtApiInterface*> m_apis;
        QList<QObject*> deleteList;

    private:
        WrtApiProvider* q; // not owned

    };


    class Q_DECL_EXPORT WrtApiProvider : public QObject, public WRT::ServiceProvider
    {
        Q_OBJECT
    public:
        /** Constructor
         */
        explicit WrtApiProvider();

        /** Destructor
         */
        virtual ~WrtApiProvider();

        /** Creates WRT API objects based set API enums. Exposes objects to JavaScript if their
         *  reimplementation of WrtWpiInterface:name() returns a non-empty
         *  string.
         */
        void handleNewInstance();

        /** Deletes all WRT APIs
         *  @param loader Loader to be used for accessing QWebPage.
         */
        void destroyInstance();

        /** Sets the apis to be loaded.
         *  @param apis Or'ed enumeration of JavaScript APIs to be loaded.
         */
        void setApis(JSApi::WidgetJavaScriptApiFlags apis);

        /** Returns the API object based on API name.
         *  @param api JavaScript API to get.
         *  @return api interface of api, if api was instantiated; otherwise 0.
         */
        WrtApiInterface* apiObject(JSApi::WidgetJavaScriptApiFlag api);

        bool addService(const QString& exposeName, QObject& object, bool giveToGarbageCollector = false);
        void setContext(WrtPage* page);
        WrtPage* getContext();

    public Q_SLOTS:
        /** Calls WrtWpiInterface::pageOnReload for each WRT API.
         */
        virtual void pageOnReload();
        void windowObjectCleared();

    private:
        WrtApiProviderPrivate* d;

        friend class WrtApiProviderPrivate;
    };

}
#endif /* WRTAPIPROVIDER_HH */



