/*
 * Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
 *
 * This file is part of Qt Web Runtime.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef __WIDGETMANAGER_P_H__
#define __WIDGETMANAGER_P_H__

#include "WidgetUtils.h"
#include "SuperWidget.h"

class QEventLoop;

class WidgetProperties;
class W3cXmlPlugin;

class WidgetManagerPrivate : public QObject
{
    Q_OBJECT

public:
    enum WidgetManagerPromptButtons {
        WidgetManagerPromptButtonYes,
        WidgetManagerPromptButtonNo,
        WidgetManagerPromptButtonOkay,
        WidgetManagerPromptButtonCancel
    };

public:
    WidgetManagerPrivate(QWidget *parent, WidgetManager*);
    ~WidgetManagerPrivate();

    WidgetProperties* getWidgetProperties(const QString &pkgPath);
    SuperWidget* createWidget(const QString &pkgPath);
    SuperWidget* getWidgetFromCache(QString pkgPath);
    void setWidgetIntoCache(QString pkgPath, SuperWidget* widget);
    QString setDefaultRootDirectory(const QString& rootDirectory);
    bool validateDriveLetter(const QString& rootDirectory = NULL);
    bool checkCallerPermissions();
    bool showQuestion(QString title,
                      QString msg,
                      WidgetManagerPromptButtons primaryButton,
                      WidgetManagerPromptButtons secondaryButton);
    void showInformation(QString title, QString msg);
    void uninstallComplete(WidgetUninstallError errorCode);

public Q_SLOTS:
    void handleInstallProgress(int p);
    void handleInstallError(WidgetInstallError error);
    void handleDestinationSelection(unsigned long spaceRequired,
                                    bool allowRemovable);
    void handleWidgetReplacement(QString title);
    void handleFeatureInstallation(QList<QString> capList);
    void handleUntrustedWidgetInstallation(WidgetInstallError errorCode);
    void handleInstallationSuccess();

public:
    void * m_data;
    W3cXmlPlugin *m_w3property;
    WidgetType m_widgetType;
    QString m_contentType;
    SuperWidget* m_installingWidget;
    QString m_rootDirectory;
    QHash<QString, SuperWidget*> m_widgetCache;

    bool m_silent;
    bool m_disableUnsignWidgetSignCheck;
    WRT::InstallationAttributes m_attributes;
    QWidget *m_parent;

public:
    struct DialogResult {
        bool m_dialogAccepted;
        QString m_resultText;
    };

    QEventLoop* m_eventLoop;
    QString m_acceptText;
    DialogResult m_dialogResult;

private:
    WidgetManager* q; // not owned

};

#endif
