# Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
#
# This file is part of Qt Web Runtime.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# version 2.1 as published by the Free Software Foundation.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
#
#

#
#  widget utilities files
#
TEMPLATE = lib
TARGET = WrtWidgetManager

QT += xml sql
CONFIG += dll link_prl create_prl

ROOT_DIR = $$PWD/../../..
WRT_DIR = $$ROOT_DIR/wrt
include($$ROOT_DIR/wrt/wrt.pri)
include($$ROOT_DIR/wrt/cwrt.pri)
include($$ROOT_DIR/qtsfw.pri)
include($$ROOT_DIR/wrt/utilities/CertificateManager/CertificateManager.pri)
include($$ROOT_DIR/wrt/utilities/W3CWidgetVerifier/W3CWidgetVerifier.pri)

DEFINES *=BUILDING_WIDGETUTILS_LIB
!symbian:include (../../3rdparty/zlib/zlib.pri)

contains ( what, blockunsignedwidgets ) {
    DEFINES += BLOCK_UNSIGNED_WIDGETS
}

contains (what, enable-widget-deps): DEFINES += ENABLE_WIDGET_DEPS

INCLUDEPATH += $$PWD \
               $$PWD/private \
               $$PWD/resource  \
               $$PWD/W3CXmlParser \
               $$PWD/../../.. \
               $$PWD/../../3rdparty/zlib \
               $$PWD/../../runtimecore \
               $$PWD/../../../src/common/securestorage \
               $$PWD/../../../src/common/securestorage/platform/qt \
               $$PWD/../../../src/common/security \
               $$PWD/../../MiscServices/SchemeServices \
               $$PWD/../../settings/inc \
               $$PWD/../../utilities/W3CWidgetVerifier

INCLUDEPATH += $$OPENSSL_INC_DIR
INCLUDEPATH += $$LIBXML2_INC_DIR

LIBS += -lWrtSecureStorage -lWrtSecurityManager -lWrtSettings

HEADERS += \
           $$PWD/private/WidgetUtilsLogs.h \
           $$PWD/private/webappmanager_p.h \
           $$PWD/private/WidgetInfo.h \
           $$PWD/W3CXmlParser/configw3xmlparser.h \
           $$PWD/W3CXmlParser/configw3xmlparser_p.h \
           $$PWD/W3CXmlParser/proprietarysettingskeys.h \
           $$PWD/W3CXmlParser/proprietarytags.h \
           $$PWD/W3CXmlParser/w3celement.h \
           $$PWD/W3CXmlParser/w3csettingskeys.h \
           $$PWD/W3CXmlParser/w3ctags.h \
           $$PWD/W3CXmlParser/w3cxmlplugin.h \
           $$PWD/AttributeMap.h \
           $$PWD/featuremapping.h \
           $$PWD/PropertyNames.h \
           $$PWD/SuperWidget.h \
           $$PWD/webappinfo.h \
           $$PWD/webapplocalizer.h \
           $$PWD/WebAppRegistry.h \
           $$PWD/WebAppRegistry_p.h \
           $$PWD/webappuniquenesshelper.h \
           $$PWD/WgtWidget.h \
           $$PWD/widgetinstaller.h \
           $$PWD/WidgetManager.h \
           $$PWD/WidgetManager_p.h \
           $$PWD/widgetmanagerconstants.h \
           $$PWD/WidgetProperties.h \
           $$PWD/WidgetUtils.h

SOURCES += \
           $$PWD/W3CXmlParser/w3celement.cpp \
           $$PWD/W3CXmlParser/w3cxmlplugin.cpp \
           $$PWD/W3CXmlParser/configw3xmlparser.cpp \
           $$PWD/WidgetManager.cpp \
           $$PWD/WidgetManager_p.cpp \
           $$PWD/WebAppRegistry.cpp \
           $$PWD/webappinfo.cpp \
           $$PWD/webapplocalizer.cpp \
           $$PWD/webappuniquenesshelper.cpp \
           $$PWD/SuperWidget.cpp \
           $$PWD/WgtWidget.cpp \
           $$PWD/featuremapping.cpp

RESOURCES += $$PWD/resources.qrc

!symbian {
    DEFINES += SHA256_SUPPORT
}

contains( what, no_sha256 ) {
    DEFINES -= SHA256_SUPPORT
}

unix:!symbian {
    CONFIG += link_pkgconfig
    PKGCONFIG += openssl libxml-2.0 libarchive

    maemo {
        DEFINES += USE_NEW_LOCALIZATION
        DEFINES += ENABLE_WBENCHMARK_ALL

        SOURCES += $$PWD/platform/maemo/widgetinstaller.cpp \
                   $$PWD/platform/maemo/widgetinstaller_p.cpp \
                   $$PWD/platform/maemo/desktopfilewriter.cpp \
                   $$PWD/platform/maemo/archiver.cpp \
                   $$PWD/platform/maemo/packageutils.cpp \
                   $$PWD/platform/maemo/debianutils.cpp

        HEADERS += $$PWD/platform/maemo/widgetinstaller_p.h \
                   $$PWD/platform/maemo/desktopfilewriter.h \
                   $$PWD/platform/maemo/archiver.h \
                   $$PWD/platform/maemo/packageutils.h \
                   $$PWD/platform/maemo/debianutils.h

        maemo5 {
            PKGCONFIG += dbus-1 libhildonmime glib-2.0
        }
    }
}

include($$WRT_DIR/cwrt-export.pri)
