/*
 * Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
 *
 * This file is part of Qt Web Runtime.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef __WRTGLOBAL_H__
#define __WRTGLOBAL_H__

/* USE() - use a library or OS service */
/* Often specific to a set - but not all - of ports to manage non-core dependencies, and are often mandatory to create a successful build for a particular port. */
#define USE(LIBRARY) (defined QTWRT_USE_##LIBRARY && QTWRT_USE_##LIBRARY)

/* ENABLE() - turn on a specific feature */
/* Port independent flags; the same flag can be used to turn on/off particular features on all ports. */
/* Disabling all feature flags should still result in a successful build in all ports. */
/* Ports should establish the default feature configuration in source tree (e.g. shared.pri). */
/* Name of the FEATURE should be a positive noun - e.g. HIERARCHICAL_MENUS instead of ONLY_FLAT_MENUS. */
#define ENABLE(FEATURE) (defined QTWRT_ENABLE_##FEATURE && QTWRT_ENABLE_##FEATURE)

#include <QFlags>

namespace WRT {
    namespace JSApi {
      enum WidgetJavaScriptApiFlag  {
          Device = 0x001,
          Utils = 0x002,
          MenuItem = 0x004,
          Menu = 0x008,
          ContextMenu = 0x010,
          WidgetInterface = 0x020,
          Version = 0x080,
          Toolbar = 0x100,
          EscapeButton = 0x200,
          Theme = 0x400,
      };
      Q_DECLARE_FLAGS(WidgetJavaScriptApiFlags, WidgetJavaScriptApiFlag)
      Q_DECLARE_OPERATORS_FOR_FLAGS(WidgetJavaScriptApiFlags)

      const WidgetJavaScriptApiFlags W3C = Menu | ContextMenu | MenuItem | WidgetInterface | Device | Version;
   }
}


#endif // __WRTGLOBAL_H__

